@@ -1,11 +1,5 @@
-##
-# Copyright 2009-2012 Ghent University
-# Copyright 2009-2012 Stijn De Weirdt
-# Copyright 2010 Dries Verdegem
-# Copyright 2010-2012 Kenneth Hoste
-# Copyright 2011 Pieter De Baets
-# Copyright 2011-2012 Jens Timmerman
-# Copyright 2012 Toon Willems
+# #
+# Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -27,14 +21,21 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 Generic EasyBuild support for building and installing software.
 The EasyBlock class should serve as a base class for all easyblocks.
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Dries Verdegem (Ghent University)
+@author: Kenneth Hoste (Ghent University)
+@author: Pieter De Baets (Ghent University)
+@author: Jens Timmerman (Ghent University)
+@author: Toon Willems (Ghent University)
 """
 
 import copy
-import grp  #@UnresolvedImport
+import grp  # @UnresolvedImport
 import re
 import os
 import shutil
@@ -42,19 +43,25 @@ import stat
 import time
 import urllib
 from distutils.version import LooseVersion
+from vsc import fancylogger
+from vsc.utils.missing import nub
 
 import easybuild.tools.environment as env
-from easybuild.framework.easyconfig import EasyConfig, get_paths_for
-from easybuild.tools.build_log import EasyBuildError, init_logger, print_msg, remove_log_handler
-from easybuild.tools.config import build_path, install_path, log_path, read_only_installdir
-from easybuild.tools.config import source_path, module_classes
+from easybuild.framework.easyconfig.easyconfig import EasyConfig, ITERATE_OPTIONS
+from easybuild.framework.easyconfig.tools import get_paths_for
+from easybuild.framework.easyconfig.templates import TEMPLATE_NAMES_EASYBLOCK_RUN_STEP
+from easybuild.tools.build_log import EasyBuildError, print_msg
+from easybuild.tools.config import build_path, install_path, log_path, get_log_filename
+from easybuild.tools.config import read_only_installdir, source_path, module_classes
 from easybuild.tools.filetools import adjust_permissions, apply_patch, convert_name, download_file
 from easybuild.tools.filetools import encode_class_name, extract_file, run_cmd, rmtree2, modify_env
 from easybuild.tools.module_generator import GENERAL_CLASS, ModuleGenerator
+from easybuild.tools.modules import ROOT_ENV_VAR_NAME_PREFIX, VERSION_ENV_VAR_NAME_PREFIX, DEVEL_ENV_VAR_NAME_PREFIX
 from easybuild.tools.modules import Modules, get_software_root
 from easybuild.tools.systemtools import get_core_count
-from easybuild.tools.version import VERBOSE_VERSION, VERSION
+from easybuild.tools.version import this_is_easybuild, VERBOSE_VERSION, VERSION
 
+_log = fancylogger.getLogger('easyblock')
 
 class EasyBlock(object):
     """Generic support for building and installing software, base class for actual easyblocks."""
@@ -97,24 +104,23 @@ class EasyBlock(object):
 
         # easyconfig for this application
         all_stops = [x[0] for x in self.get_steps()]
-        self.cfg = EasyConfig(path, extra_options=self.extra_options(), valid_module_classes=module_classes(),
-                              valid_stops=all_stops)
+        self.cfg = EasyConfig(path,
+                              extra_options=self.extra_options(),
+                              valid_module_classes=module_classes(),
+                              valid_stops=all_stops
+                              )
 
         # module generator
         self.moduleGenerator = None
 
-        self.sanityCheckOK = False
-
         # indicates whether build should be performed in installation dir
         self.build_in_installdir = False
 
         # logging
         self.log = None
         self.logfile = None
-        self.loghandler = None
         self.logdebug = debug
         self.postmsg = ''  # allow a post message to be set, which can be shown as last output
-        self.init_log()
 
         # original environ will be set later
         self.orig_environ = {}
@@ -128,23 +134,37 @@ class EasyBlock(object):
         # original module path
         self.orig_modulepath = os.getenv('MODULEPATH')
 
+        # at the end of __init__, initialise the logging
+        self._init_log()
+
+        # iterate configure/build/options
+        self.iter_opts = {}
+
+        self.log.info("Init completed for application name %s version %s" % (self.name, self.version))
+
+
     # INIT/CLOSE LOG
-    def init_log(self):
+    def _init_log(self):
         """
         Initialize the logger.
         """
-        if not self.log:
-            self.logfile, self.log, self.loghandler = init_logger(self.name, self.version,
-                                                                  self.logdebug, typ=self.__class__.__name__)
-            self.log.info("Init completed for application name %s version %s" % (self.name, self.version))
+        if not self.log is None:
+            return
+
+        self.logfile = get_log_filename(self.name, self.version)
+        fancylogger.logToFile(self.logfile)
+
+        self.log = fancylogger.getLogger(name=self.__class__.__name__, fname=False)
+
+        self.log.info(this_is_easybuild())
+
 
     def close_log(self):
         """
         Shutdown the logger.
         """
         self.log.info("Closing log for application name %s version %s" % (self.name, self.version))
-        remove_log_handler(self.loghandler)
-        self.loghandler.close()
+        fancylogger.logToFile(self.logfile, enable=False)
 
 
     #
@@ -158,10 +178,10 @@ class EasyBlock(object):
         """
 
         for src_entry in list_of_sources:
-            if type(src_entry) in [list, tuple]:
+            if isinstance(src_entry, (list, tuple)):
                 cmd = src_entry[1]
                 source = src_entry[0]
-            elif type(src_entry) == str:
+            elif isinstance(src_entry, basestring):
                 cmd = None
                 source = src_entry
 
@@ -184,18 +204,18 @@ class EasyBlock(object):
         patches = []
         for patchFile in list_of_patches:
 
-            ## check if the patches can be located
+            # check if the patches can be located
             copy_file = False
             suff = None
             level = None
-            if type(patchFile) in [list, tuple]:
+            if isinstance(patchFile, (list, tuple)):
                 if not len(patchFile) == 2:
                     self.log.error("Unknown patch specification '%s', only two-element lists/tuples are supported!" % patchFile)
                 pf = patchFile[0]
 
-                if type(patchFile[1]) == int:
+                if isinstance(patchFile[1], int):
                     level = patchFile[1]
-                elif type(patchFile[1]) == str:
+                elif isinstance(patchFile[1], basestring):
                     # non-patch files are assumed to be files to copy
                     if not patchFile[0].endswith('.patch'):
                         copy_file = True
@@ -235,7 +255,10 @@ class EasyBlock(object):
         Find source file for extensions.
         """
         exts_sources = []
-        for ext in self.cfg['exts_list']:
+        self.cfg.enable_templating = False
+        exts_list = self.cfg['exts_list']
+        self.cfg.enable_templating = True
+        for ext in exts_list:
             if (isinstance(ext, list) or isinstance(ext, tuple)) and ext:
 
                 # expected format: (name, version, options (dict))
@@ -305,9 +328,9 @@ class EasyBlock(object):
         srcpath = source_path()
 
         # make sure we always deal with a list, to avoid code duplication
-        if type(srcpath) == str:
+        if isinstance(srcpath, basestring):
             srcpaths = [srcpath]
-        elif type(srcpath) == list:
+        elif isinstance(srcpath, list):
             srcpaths = srcpath
         else:
             self.log.error("Source path '%s' has incorrect type: %s" % (srcpath, type(srcpath)))
@@ -361,7 +384,7 @@ class EasyBlock(object):
             common_filepaths = []
             if not self.robot_path is None:
                 common_filepaths.append(self.robot_path)
-            common_filepaths.extend(get_paths_for(self.log, "easyconfigs", robot_path=self.robot_path))
+            common_filepaths.extend(get_paths_for("easyconfigs", robot_path=self.robot_path))
 
             for path in common_filepaths + srcpaths:
                 # create list of candidate filepaths
@@ -384,7 +407,7 @@ class EasyBlock(object):
                     if extension:
                         fullpaths = [
                                      os.path.join(cfp, "extensions", filename),
-                                     os.path.join(cfp, "packages", filename), # legacy
+                                     os.path.join(cfp, "packages", filename),  # legacy
                                      fullpath
                                     ]
                     else:
@@ -394,12 +417,12 @@ class EasyBlock(object):
                         if os.path.isfile(fp):
                             self.log.info("Found file %s at %s" % (filename, fp))
                             foundfile = os.path.abspath(fp)
-                            break # no need to try further
+                            break  # no need to try further
                         else:
                             failedpaths.append(fp)
 
                 if foundfile:
-                    break # no need to try other source paths
+                    break  # no need to try other source paths
 
             if foundfile:
                 return foundfile
@@ -425,14 +448,14 @@ class EasyBlock(object):
                     else:
                         targetpath = os.path.join(targetdir, filename)
 
-                    if type(url) == str:
+                    if isinstance(url, basestring):
                         if url[-1] in ['=', '/']:
                             fullurl = "%s%s" % (url, filename)
                         else:
                             fullurl = "%s/%s" % (url, filename)
-                    elif type(url) == tuple:
-                        ## URLs that require a suffix, e.g., SourceForge download links
-                        ## e.g. http://sourceforge.net/projects/math-atlas/files/Stable/3.8.4/atlas3.8.4.tar.bz2/download
+                    elif isinstance(url, tuple):
+                        # URLs that require a suffix, e.g., SourceForge download links
+                        # e.g. http://sourceforge.net/projects/math-atlas/files/Stable/3.8.4/atlas3.8.4.tar.bz2/download
                         fullurl = "%s/%s/%s" % (url[0], filename, url[1])
                     else:
                         self.log.warning("Source URL %s is of unknown type, so ignoring it." % url)
@@ -494,7 +517,7 @@ class EasyBlock(object):
         """
         if not self.build_in_installdir:
             # make a unique build dir
-            ## if a tookitversion starts with a -, remove the - so prevent a -- in the path name
+            # if a tookitversion starts with a -, remove the - so prevent a -- in the path name
             tcversion = self.toolchain.version
             if tcversion.startswith('-'):
                 tcversion = tcversion[1:]
@@ -581,7 +604,7 @@ class EasyBlock(object):
             self.log.info("Cleaning only, no actual creation of %s, only verification/creation of dirname %s" % (olddir, dirName))
             if os.path.exists(dirName):
                 return
-            ## if not, create dir as usual
+            # if not, create dir as usual
 
         try:
             os.makedirs(dirName)
@@ -621,7 +644,9 @@ class EasyBlock(object):
         # these should be all the dependencies and we should load them
         for key in os.environ:
             # legacy support
-            if key.startswith("EBDEVEL") or key.startswith("SOFTDEVEL"):
+            if key.startswith(DEVEL_ENV_VAR_NAME_PREFIX) or key.startswith("SOFTDEVEL"):
+                if key.startswith("SOFTDEVEL"):
+                    self.log.deprecated("Environment variable SOFTDEVEL* being relied on", "2.0")
                 if not key.endswith(convert_name(self.name, upper=True)):
                     path = os.environ[key]
                     if os.path.isfile(path):
@@ -688,11 +713,11 @@ class EasyBlock(object):
 
         # EBROOT + EBVERSION + EBDEVEL
         environment_name = convert_name(self.name, upper=True)
-        txt += self.moduleGenerator.set_environment("EBROOT" + environment_name, "$root")
-        txt += self.moduleGenerator.set_environment("EBVERSION" + environment_name, self.version)
+        txt += self.moduleGenerator.set_environment(ROOT_ENV_VAR_NAME_PREFIX + environment_name, "$root")
+        txt += self.moduleGenerator.set_environment(VERSION_ENV_VAR_NAME_PREFIX + environment_name, self.version)
         devel_path = os.path.join("$root", log_path(), "%s-%s-easybuild-devel" % (self.name,
             self.get_installversion()))
-        txt += self.moduleGenerator.set_environment("EBDEVEL" + environment_name, devel_path)
+        txt += self.moduleGenerator.set_environment(DEVEL_ENV_VAR_NAME_PREFIX + environment_name, devel_path)
 
         txt += "\n"
         for (key, value) in self.cfg['modextravars'].items():
@@ -733,18 +758,22 @@ class EasyBlock(object):
         A dictionary of possible directories to look for.
         """
         return {
-            'PATH': ['bin'],
+            'PATH': ['bin', 'sbin'],
             'LD_LIBRARY_PATH': ['lib', 'lib64'],
             'CPATH':['include'],
             'MANPATH': ['man', 'share/man'],
             'PKG_CONFIG_PATH' : ['lib/pkgconfig', 'share/pkgconfig'],
         }
 
-    def load_module(self):
+    def load_module(self, mod_paths=None, purge=True):
         """
-        Load module for this software package/version.
+        Load module for this software package/version, after purging all currently loaded modules.
         """
-        m = Modules()
+        m = Modules(mod_paths)
+        # purge all loaded modules if desired
+        if purge:
+            m.purge()
+        m.check_module_path()  # make sure MODULEPATH is set correctly after purging
         m.add_module([[self.name, self.get_installversion()]])
         m.load()
 
@@ -753,7 +782,7 @@ class EasyBlock(object):
         Create and load fake module.
         """
 
-        # take a copy of the environment before loading the fake module
+        # take a copy of the environment before loading the fake module, so we can restore it
         orig_env = copy.deepcopy(os.environ)
 
         # make fake module
@@ -762,19 +791,9 @@ class EasyBlock(object):
         # create Modules instance
         mod_paths = [fake_mod_path]
         mod_paths.extend(self.orig_modulepath.split(':'))
-        m = Modules(mod_paths)
         self.log.debug("mod_paths: %s" % mod_paths)
 
-        # purge loaded modules if desired
-        if purge:
-            m.purge()
-
-        # make sure MODULEPATH is set correctly after purging
-        m.check_module_path()
-
-        # load the module
-        m.add_module([[self.name, self.get_installversion()]])
-        m.load()
+        self.load_module(mod_paths=mod_paths, purge=purge)
 
         return (fake_mod_path, orig_env)
 
@@ -797,13 +816,8 @@ class EasyBlock(object):
             except OSError, err:
                 self.log.error("Failed to clean up fake module dir: %s" % err)
 
-        # restore env vars that were set before loading the fake module,
-        # but were unset after unloading it because the module also sets them
-        curr_env = copy.deepcopy(os.environ)
-        for (var, val) in orig_env.items():
-            if not var in curr_env:
-                os.environ[var] = val
-                self.log.debug("Restored value %s for %s that was unset by unloading fake module." % (val, var))
+        # restore original environment
+        modify_env(os.environ, orig_env)
 
     #
     # EXTENSIONS UTILITY FUNCTIONS
@@ -821,7 +835,11 @@ class EasyBlock(object):
         - use this to detect existing extensions and to remove them from self.exts
         - based on initial R version
         """
+        # disabling templating is required here to support legacy string templates like name/version
+        self.cfg.enable_templating = False
         exts_filter = self.cfg['exts_filter']
+        self.cfg.enable_templating = True
+
         if not exts_filter or len(exts_filter) == 0:
             self.log.error("Skipping of extensions, but no exts_filter set in easyconfig")
         elif isinstance(exts_filter, basestring) or len(exts_filter) != 2:
@@ -839,9 +857,14 @@ class EasyBlock(object):
             else:
                 modname = name
             tmpldict = {
+                        'ext_name': modname,
+                        'ext_version': ext.get('version'),
+                        'src': ext.get('source'),
+                        # the ones below are only there for legacy purposes
+                        # TODO deprecated, remove in v2.0
+                        # TODO same dict is used in extension.py sanity_check_step, resolve this
                         'name': modname,
                         'version': ext.get('version'),
-                        'src': ext.get('source')
                        }
             cmd = cmdtmpl % tmpldict
             if cmdinputtmpl:
@@ -908,6 +931,41 @@ class EasyBlock(object):
         except OSError, err:
             self.log.exception("Can't change to real build directory %s: %s" % (self.cfg['start_dir'], err))
 
+    def handle_iterate_opts(self):
+        """Handle options relevant during iterated part of build/install procedure."""
+
+        # disable templating in this function, since we're messing about with values in self.cfg
+        self.cfg.enable_templating = False
+
+        # handle configure/build/install options that are specified as lists
+        # set first element to be used, keep track of list in *_list options dictionary
+        # this will only be done during first iteration, since after that the options won't be lists anymore
+        suffix = "_list"
+        sufflen = len(suffix)
+        for opt in ITERATE_OPTIONS:
+            # keep track of list, supply first element as first option to handle
+            if isinstance(self.cfg[opt], (list, tuple)):
+                self.iter_opts[opt + suffix] = self.cfg[opt]  # copy
+                self.log.debug("Found list for %s: %s" % (opt, self.iter_opts[opt + suffix]))
+
+        # pop first element from all *_list options as next value to use
+        for (lsname, ls) in self.iter_opts.items():
+            opt = lsname[:-sufflen]  # drop '_list' part from name to get option name
+            if len(self.iter_opts[lsname]) > 0:
+                self.cfg[opt] = self.iter_opts[lsname].pop(0)  # first element will be used next
+            else:
+                self.cfg[opt] = ''  # empty list => empty option as next value
+            self.log.debug("Next value for %s: %s" % (opt, str(self.cfg[opt])))
+
+        # re-enable templating before self.cfg values are used
+        self.cfg.enable_templating = True
+
+    def det_iter_cnt(self):
+        """Determine iteration count based on configure/build/install options that may be lists."""
+        iter_cnt = max([1] + [len(self.cfg[opt]) for opt in ITERATE_OPTIONS
+                              if isinstance(self.cfg[opt], (list, tuple))])
+        return iter_cnt
+
     def print_environ(self):
         """
         Prints the environment changes and loaded modules to the debug log
@@ -952,13 +1010,13 @@ class EasyBlock(object):
                 self.log.error("Parallelism %s not integer: %s" % (nr, err))
         else:
             nr = get_core_count()
-            ## check ulimit -u
+            # check ulimit -u
             out, ec = run_cmd('ulimit -u')
             try:
                 if out.startswith("unlimited"):
                     out = 2 ** 32 - 1
                 maxuserproc = int(out)
-                ## assume 6 processes per build thread + 15 overhead
+                # assume 6 processes per build thread + 15 overhead
                 maxnr = int((maxuserproc - 15) / 6)
                 if maxnr < nr:
                     nr = maxnr
@@ -1037,12 +1095,18 @@ class EasyBlock(object):
                 self.log.info("No current version (name: %s, version: %s) found. Not skipping anything." % (self.name,
                     self.get_installversion()))
 
+        # Set group id, if a group was specified
+        if self.cfg['group']:
+            gid = grp.getgrnam(self.cfg['group'])[2]
+            os.setgid(gid)
+            self.log.debug("Changing group to %s (gid: %s)" % (self.cfg['group'], gid))
+
     def fetch_step(self):
         """
         prepare for building
         """
 
-        ## check EasyBuild version
+        # check EasyBuild version
         easybuild_version = self.cfg['easybuild_version']
         if not easybuild_version:
             self.log.warn("Easyconfig does not specify an EasyBuild-version (key 'easybuild_version')! Assuming the latest version")
@@ -1108,7 +1172,7 @@ class EasyBlock(object):
             self.log.info("Applying patch %s" % tmp['name'])
 
             copy = False
-            ## default: patch first source
+            # default: patch first source
             srcind = 0
             if 'source' in tmp:
                 srcind = tmp['source']
@@ -1355,34 +1419,31 @@ class EasyBlock(object):
         # check sanity check paths
         ks = paths.keys()
         ks.sort()
-        valnottypes = [type(x) != list for x in paths.values()]
+        valnottypes = [not isinstance(x, list) for x in paths.values()]
         lenvals = [len(x) for x in paths.values()]
         if not ks == ["dirs", "files"] or sum(valnottypes) > 0 or sum(lenvals) == 0:
             self.log.error("Incorrect format for sanity_check_paths (should only have 'files' and 'dirs' keys, " \
                            "values should be lists (at least one non-empty)).")
 
-        self.sanityCheckOK = True
+        self.sanity_check_fail_msgs = []
 
         # check if files exist
         for f in paths['files']:
             p = os.path.join(self.installdir, f)
             if not os.path.exists(p):
-                self.log.debug("Sanity check: did not find file %s in %s" % (f, self.installdir))
-                self.sanityCheckOK = False
-                break
+                self.sanity_check_fail_msgs.append("did not find file %s in %s" % (f, self.installdir))
+                self.log.warning("Sanity check: %s" % self.sanity_check_fail_msgs[-1])
             else:
                 self.log.debug("Sanity check: found file %s in %s" % (f, self.installdir))
 
-        if self.sanityCheckOK:
-            # check if directories exist, and whether they are non-empty
-            for d in paths['dirs']:
-                p = os.path.join(self.installdir, d)
-                if not os.path.isdir(p) or not os.listdir(p):
-                    self.log.debug("Sanity check: did not find non-empty directory %s in %s" % (d, self.installdir))
-                    self.sanityCheckOK = False
-                    break
-                else:
-                    self.log.debug("Sanity check: found non-empty directory %s in %s" % (d, self.installdir))
+        # check if directories exist, and whether they are non-empty
+        for d in paths['dirs']:
+            p = os.path.join(self.installdir, d)
+            if not os.path.isdir(p) or not os.listdir(p):
+                self.sanity_check_fail_msgs.append("did not find non-empty directory %s in %s" % (d, self.installdir))
+                self.log.warning("Sanity check: %s" % self.sanity_check_fail_msgs[-1])
+            else:
+                self.log.debug("Sanity check: found non-empty directory %s in %s" % (d, self.installdir))
 
         fake_mod_data = None
         try:
@@ -1390,8 +1451,8 @@ class EasyBlock(object):
             # this ensures that loading of dependencies is tested, and avoids conflicts with build dependencies
             fake_mod_data = self.load_fake_module(purge=True)
         except EasyBuildError, err:
-            self.log.info("Loading fake module failed: %s" % err)
-            self.sanityCheckOK = False
+            self.sanity_check_fail_msgs.append("loading fake module failed: %s" % err)
+            self.log.warning("Sanity check: %s" % self.sanity_check_fail_msgs[-1])
 
         # chdir to installdir (better environment for running tests)
         os.chdir(self.installdir)
@@ -1426,24 +1487,24 @@ class EasyBlock(object):
 
             out, ec = run_cmd(cmd, simple=False, log_ok=False, log_all=False)
             if ec != 0:
-                self.sanityCheckOK = False
-                self.log.warning("sanityCheckCommand %s exited with code %s (output: %s)" % (cmd, ec, out))
+                self.sanity_check_fail_msgs.append("sanity check command %s exited with code %s (output: %s)" % (cmd, ec, out))
+                self.log.warning("Sanity check: %s" % self.sanity_check_fail_msgs[-1])
             else:
-                self.log.info("sanityCheckCommand %s ran successfully! (output: %s)" % (cmd, out))
+                self.log.debug("sanity check command %s ran successfully! (output: %s)" % (cmd, out))
 
         failed_exts = [ext.name for ext in self.ext_instances if not ext.sanity_check_step()]
 
         if failed_exts:
-            self.log.info("Sanity check for extensions %s failed!" % failed_exts)
-            self.sanityCheckOK = False
+            self.sanity_check_fail_msgs.append("sanity checks for %s extensions failed!" % failed_exts)
+            self.log.warning("Sanity check: %s" % self.sanity_check_fail_msgs[-1])
 
         # cleanup
         if fake_mod_data:
             self.clean_up_fake_module(fake_mod_data)
 
         # pass or fail
-        if not self.sanityCheckOK:
-            self.log.error("Sanity check failed!")
+        if self.sanity_check_fail_msgs:
+            self.log.error("Sanity check failed: %s" % ', '.join(self.sanity_check_fail_msgs))
         else:
             self.log.debug("Sanity check passed!")
 
@@ -1520,14 +1581,24 @@ class EasyBlock(object):
             except EasyBuildError, err:
                 self.log.exception("Running test %s failed: %s" % (path, err))
 
+    def update_config_template_run_step(self):
+        """Update the the easyconfig template dictionary with easyconfig.TEMPLATE_NAMES_EASYBLOCK_RUN_STEP names"""
+
+        for name in TEMPLATE_NAMES_EASYBLOCK_RUN_STEP:
+            self.cfg.template_values[name[0]] = str(getattr(self, name[0], None))
+        self.cfg.generate_template_values()
+
     def run_step(self, step, methods, skippable=False):
         """
         Run step, returns false when execution should be stopped
         """
-        if skippable and self.skip:
+        if skippable and (self.skip or step in self.cfg['skipsteps']):
             self.log.info("Skipping %s step" % step)
         else:
             self.log.info("Starting %s step" % step)
+            # update the config templates
+            self.update_config_template_run_step()
+
             for m in methods:
                 self.log.info("Running method %s part of step %s" % ('_'.join(m.func_code.co_names), step))
                 m(self)
@@ -1537,36 +1608,91 @@ class EasyBlock(object):
             raise StopException(step)
 
     @staticmethod
-    def get_steps(run_test_cases=True):
+    def get_steps(run_test_cases=True, iteration_count=1):
         """Return a list of all steps to be performed."""
 
-        steps = [
-                  # stop name: (description, list of functions, skippable)
-                  ('fetch', 'fetching files', [lambda x: x.fetch_step()], False),
-                  ('ready', "getting ready, creating build dir, resetting environment",
-                   [lambda x: x.check_readiness_step(), lambda x: x.gen_installdir(),
-                    lambda x: x.make_builddir(), lambda x: env.reset_changes()],
-                   False),
-                  ('source', 'unpacking', [lambda x: x.checksum_step(),
-                                           lambda x: x.extract_step()], True),
-                  ('patch', 'patching', [lambda x: x.patch_step()], True),
-                  ('prepare', 'preparing', [lambda x: x.prepare_step()], False),
-                  ('configure', 'configuring', [lambda x: x.configure_step()], True),
-                  ('build', 'building', [lambda x: x.build_step()], True),
-                  ('test', 'testing', [lambda x: x.test_step()], True),
-                  ('install', 'installing', [
-                                             lambda x: x.stage_install_step(),
-                                             lambda x: x.make_installdir(),
-                                             lambda x: x.install_step(),
-                                             ],
-                   True),
-                  ('extensions', 'taking care of extensions', [lambda x: x.extensions_step()], False),
-                  ('package', 'packaging', [lambda x: x.package_step()], True),
-                  ('postproc', 'postprocessing', [lambda x: x.post_install_step()], True),
-                  ('sanitycheck', 'sanity checking', [lambda x: x.sanity_check_step()], False),
-                  ('cleanup', 'cleaning up', [lambda x: x.cleanup_step()], False),
-                  ('module', 'creating module', [lambda x: x.make_module_step()], False),
-                  ]
+        def get_step(tag, descr, substeps, skippable, initial=True):
+            """Determine step definition based on whether it's an initial run or not."""
+            substeps = [substep for (always_include, substep) in substeps if (initial or always_include)]
+            return (tag, descr, substeps, skippable)
+
+        # list of substeps for steps that are slightly different from 2nd iteration onwards
+        ready_substeps = [
+            (False, lambda x: x.check_readiness_step()),
+            (False, lambda x: x.gen_installdir()),
+            (True, lambda x: x.make_builddir()),
+            (True, lambda x: env.reset_changes()),
+            (True, lambda x: x.handle_iterate_opts()),
+        ]
+        ready_step_spec = lambda initial: get_step('ready', "creating build dir, resetting environment",
+                                                   ready_substeps, False, initial=initial)
+
+        source_substeps = [
+                           (False, lambda x: x.checksum_step()),
+                           (True, lambda x: x.extract_step()),
+                          ]
+        source_step_spec = lambda initial: get_step('source', "unpacking", source_substeps, True, initial=initial)
+
+        def prepare_step_spec(initial):
+            """Return prepare step specification."""
+            if initial:
+                substeps = [lambda x: x.prepare_step()]
+            else:
+                substeps = [lambda x: x.guess_start_dir()]
+            return ('prepare', 'preparing', substeps, False)
+
+        install_substeps = [
+            (False, lambda x: x.stage_install_step()),
+            (False, lambda x: x.make_installdir()),
+            (True, lambda x: x.install_step()),
+        ]
+        install_step_spec = lambda initial: get_step('install', "installing", install_substeps, True, initial=initial)
+
+        # format for step specifications: (stop_name: (description, list of functions, skippable))
+
+        # core steps that are part of the iterated loop
+        patch_step_spec = ('patch', 'patching', [lambda x: x.patch_step()], True)
+        configure_step_spec = ('configure', 'configuring', [lambda x: x.configure_step()], True)
+        build_step_spec = ('build', 'building', [lambda x: x.build_step()], True)
+        test_step_spec = ('test', 'testing', [lambda x: x.test_step()], True)
+
+        # part 1: pre-iteration + first iteration
+        steps_part1 = [
+            ('fetch', 'fetching files', [lambda x: x.fetch_step()], False),
+            ready_step_spec(True),
+            source_step_spec(True),
+            patch_step_spec,
+            prepare_step_spec(True),
+            configure_step_spec,
+            build_step_spec,
+            test_step_spec,
+            install_step_spec(True),
+        ]
+        # part 2: iterated part, from 2nd iteration onwards
+        # repeat core procedure again depending on specified iteration count
+        # not all parts of all steps need to be rerun (see e.g., ready, prepare)
+        steps_part2 = [
+            ready_step_spec(False),
+            source_step_spec(False),
+            patch_step_spec,
+            prepare_step_spec(False),
+            configure_step_spec,
+            build_step_spec,
+            test_step_spec,
+            install_step_spec(False),
+        ] * (iteration_count - 1)
+        # part 3: post-iteration part
+        steps_part3 = [
+            ('extensions', 'taking care of extensions', [lambda x: x.extensions_step()], False),
+            ('package', 'packaging', [lambda x: x.package_step()], True),
+            ('postproc', 'postprocessing', [lambda x: x.post_install_step()], True),
+            ('sanitycheck', 'sanity checking', [lambda x: x.sanity_check_step()], False),
+            ('cleanup', 'cleaning up', [lambda x: x.cleanup_step()], False),
+            ('module', 'creating module', [lambda x: x.make_module_step()], False),
+        ]
+
+        # full list of steps, included iterated steps
+        steps = steps_part1 + steps_part2 + steps_part3
 
         if run_test_cases:
             steps.append(('testcases', 'running test cases', [
@@ -1585,7 +1711,7 @@ class EasyBlock(object):
         if self.cfg['stop'] and self.cfg['stop'] == 'cfg':
             return True
 
-        steps = self.get_steps(run_test_cases)
+        steps = self.get_steps(run_test_cases=run_test_cases, iteration_count=self.det_iter_cnt())
 
         try:
             full_name = "%s-%s" % (self.name, self.get_installversion())
@@ -1601,6 +1727,7 @@ class EasyBlock(object):
         return True
 
 
+
 def get_class_for(modulepath, class_name):
     """
     Get class for a given class name and easyblock module path.
@@ -1633,9 +1760,9 @@ def get_module_path(easyblock, generic=False):
 
     # construct character translation table for module name
     # only 0-9, a-z, A-Z are retained, everything else is mapped to _
-    charmap = 48 * '_' + ''.join([chr(x) for x in range(48, 58)]) # 0-9
-    charmap += 7 * '_' + ''.join([chr(x) for x in range(65, 91)]) # A-Z
-    charmap += 6 * '_' + ''.join([chr(x) for x in range(97, 123)]) + 133 * '_' # a-z
+    charmap = 48 * '_' + ''.join([chr(x) for x in range(48, 58)])  # 0-9
+    charmap += 7 * '_' + ''.join([chr(x) for x in range(65, 91)])  # A-Z
+    charmap += 6 * '_' + ''.join([chr(x) for x in range(97, 123)]) + 133 * '_'  # a-z
 
     module_name = easyblock.translate(charmap)
 
@@ -1646,7 +1773,7 @@ def get_module_path(easyblock, generic=False):
 
     return '.'.join([modpath, module_name.lower()])
 
-def get_class(easyblock, log, name=None):
+def get_class(easyblock, name=None):
     """
     Get class for a particular easyblock (or ConfigureMake by default)
     """
@@ -1665,22 +1792,22 @@ def get_class(easyblock, log, name=None):
 
             # try and find easyblock
             try:
-                log.debug("getting class for %s.%s" % (modulepath, class_name))
+                _log.debug("getting class for %s.%s" % (modulepath, class_name))
                 cls = get_class_for(modulepath, class_name)
-                log.info("Successfully obtained %s class instance from %s" % (class_name, modulepath))
+                _log.info("Successfully obtained %s class instance from %s" % (class_name, modulepath))
                 return cls
             except ImportError, err:
 
                 # when an ImportError occurs, make sure that it's caused by not finding the easyblock module,
                 # and not because of a broken import statement in the easyblock module
-                error_re = re.compile("No module named %s" % modulepath.replace("easybuild.easyblocks.", ''))
-                log.debug("error regexp: %s" % error_re.pattern)
+                error_re = re.compile(r"No module named %s" % modulepath.replace("easybuild.easyblocks.", ''))
+                _log.debug("error regexp: %s" % error_re.pattern)
                 if not error_re.match(str(err)):
-                    log.error("Failed to import easyblock for %s because of module issue: %s" % (class_name, err))
+                    _log.error("Failed to import easyblock for %s because of module issue: %s" % (class_name, err))
 
                 else:
                     # no easyblock could be found, so fall back to default class.
-                    log.warning("Failed to import easyblock for %s, falling back to default class %s: error: %s" % \
+                    _log.warning("Failed to import easyblock for %s, falling back to default class %s: error: %s" % \
                                 (class_name, app_mod_class, err))
                     (modulepath, class_name) = app_mod_class
                     cls = get_class_for(modulepath, class_name)
@@ -1690,7 +1817,7 @@ def get_class(easyblock, log, name=None):
             class_name = easyblock.split('.')[-1]
             # figure out if full path was specified or not
             if len(easyblock.split('.')) > 1:
-                log.info("Assuming that full easyblock module path was specified.")
+                _log.info("Assuming that full easyblock module path was specified.")
                 modulepath = '.'.join(easyblock.split('.')[:-1])
                 cls = get_class_for(modulepath, class_name)
             else:
@@ -1702,14 +1829,13 @@ def get_class(easyblock, log, name=None):
                     # we might be dealing with a non-generic easyblock, e.g. with --easyblock is used
                     modulepath = get_module_path(easyblock)
                     cls = get_class_for(modulepath, class_name)
-                log.info("Derived full easyblock module path for %s: %s" % (class_name, modulepath))
+                _log.info("Derived full easyblock module path for %s: %s" % (class_name, modulepath))
 
-        log.info("Successfully obtained %s class instance from %s" % (class_name, modulepath))
+        _log.info("Successfully obtained %s class instance from %s" % (class_name, modulepath))
         return cls
 
     except Exception, err:
-        log.error("Failed to obtain class for %s easyblock (not available?): %s" % (easyblock, err))
-        raise EasyBuildError(str(err))
+        _log.error("Failed to obtain class for %s easyblock (not available?): %s" % (easyblock, err))
 
 class StopException(Exception):
     """
