@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-##
+# #
 # Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
@@ -22,7 +22,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 Main entry point for EasyBuild: build software from .eb input file
 
@@ -48,6 +48,7 @@ import traceback
 import xml.dom.minidom as xml
 from datetime import datetime
 from vsc import fancylogger
+from vsc.utils.missing import any
 
 # optional Python packages, these might be missing
 # failing imports are just ignored
@@ -74,25 +75,30 @@ try:
 except ImportError, err:
     graph_errors.append("Failed to import graphviz: try yum install graphviz-python, or apt-get install python-pygraphviz")
 
+# IMPORTANT this has to be the first easybuild import as it customises the logging
+#  expect missing log output when this not the case!
+from easybuild.tools.build_log import  EasyBuildError, print_msg, print_error, print_warning
+
 import easybuild.framework.easyconfig as easyconfig
 import easybuild.tools.config as config
 import easybuild.tools.filetools as filetools
 import easybuild.tools.options as eboptions
 import easybuild.tools.parallelbuild as parbuild
 from easybuild.framework.easyblock import EasyBlock, get_class
-from easybuild.framework.easyconfig import EasyConfig, get_paths_for
+from easybuild.framework.easyconfig.easyconfig import EasyConfig, ITERATE_OPTIONS
+from easybuild.framework.easyconfig.tools import get_paths_for
 from easybuild.tools import systemtools
-from easybuild.tools.build_log import  EasyBuildError, print_msg, print_error, print_warning
 from easybuild.tools.version import this_is_easybuild, FRAMEWORK_VERSION, EASYBLOCKS_VERSION  # from a single location
 from easybuild.tools.config import get_repository, module_classes
 from easybuild.tools.filetools import modify_env
 from easybuild.tools.modules import Modules, search_module
 from easybuild.tools.modules import curr_module_paths, mk_module_path
 from easybuild.tools.ordereddict import OrderedDict
-from easybuild.tools.utilities import any
+
 
 log = None
 
+
 def main(testing_data=(None, None)):
     """
     Main function:
@@ -113,7 +119,9 @@ def main(testing_data=(None, None)):
     args, logfile = testing_data
 
     # initialise options
-    (options, orig_paths, opt_parser, cmd_args) = eboptions.parse_options(args=args)
+    eb_go = eboptions.parse_options(args=args)
+    options = eb_go.options
+    orig_paths = eb_go.args
 
     # initialise logging for main
     if options.logtostdout:
@@ -147,6 +155,7 @@ def main(testing_data=(None, None)):
     easyconfigs_paths = get_paths_for("easyconfigs", robot_path=options.robot)
     easyconfigs_pkg_full_path = None
 
+    search_path = os.getcwd()
     if easyconfigs_paths:
         easyconfigs_pkg_full_path = easyconfigs_paths[0]
         if not options.robot:
@@ -159,12 +168,8 @@ def main(testing_data=(None, None)):
     if options.robot:
         easyconfigs_paths = [options.robot] + easyconfigs_paths
 
-    configOptions = {}
-    if options.pretend:
-        configOptions['install_path'] = os.path.join(os.environ['HOME'], 'easybuildinstall')
-
-    # default location of configfile is set as default in the config option
-    config.init(options.config, **configOptions)
+    # initialise the easybuild configuration
+    config.init(options, eb_go.get_options_by_section('config'))
 
     # search for modules
     if options.search:
@@ -182,7 +187,7 @@ def main(testing_data=(None, None)):
         elif not any([options.aggregate_regtest, options.search, options.regtest]):
             print_error(("Please provide one or multiple easyconfig files, or use software build "
                   "options to make EasyBuild search for easyconfigs"),
-                  log=log, opt_parser=opt_parser, exit_on_error=not testing)
+                  log=log, opt_parser=eb_go.parser, exit_on_error=not testing)
     else:
         # look for easyconfigs with relative paths in easybuild-easyconfigs package,
         # unless they we found at the given relative paths
@@ -195,12 +200,12 @@ def main(testing_data=(None, None)):
                     easyconfigs_map.update({filename: os.path.join(subpath, filename)})
 
             # try and find non-existing non-absolute eaysconfig paths in easybuild-easyconfigs package install path
-            for i in range(len(orig_paths)):
-                if not os.path.isabs(orig_paths[i]) and not os.path.exists(orig_paths[i]):
-                    if orig_paths[i] in easyconfigs_map:
-                        log.info("Found %s in %s: %s" % (orig_paths[i], easyconfigs_pkg_full_path,
-                                                         easyconfigs_map[orig_paths[i]]))
-                        orig_paths[i] = easyconfigs_map[orig_paths[i]]
+            for idx, orig_path in enumerate(orig_paths):
+                if not os.path.isabs(orig_path) and not os.path.exists(orig_path):
+                    if orig_path in easyconfigs_map:
+                        log.info("Found %s in %s: %s" % (orig_path, easyconfigs_pkg_full_path,
+                                                         easyconfigs_map[orig_path]))
+                        orig_paths[idx] = easyconfigs_map[orig_path]
 
         # indicate that specified paths do not contain generated easyconfig files
         paths = [(path, False) for path in orig_paths]
@@ -243,7 +248,7 @@ def main(testing_data=(None, None)):
             files = find_easyconfigs(path)
             for f in files:
                 if not generated and try_to_generate and software_build_specs:
-                    ec_file = easyconfig.tweak(f, None, software_build_specs)
+                    ec_file = easyconfig.tools.tweak(f, None, software_build_specs)
                 else:
                     ec_file = f
                 easyconfigs.extend(process_easyconfig(ec_file, options.only_blocks,
@@ -257,19 +262,7 @@ def main(testing_data=(None, None)):
 
     # skip modules that are already installed unless forced
     if not options.force:
-        m = Modules()
-        easyconfigs, check_easyconfigs = [], easyconfigs
-        for ec in check_easyconfigs:
-            module = ec['module']
-            mod = "%s (version %s)" % (module[0], module[1])
-            modspath = mk_module_path(curr_module_paths() + [os.path.join(config.install_path("mod"), 'all')])
-            if m.exists(module[0], module[1], modspath):
-                msg = "%s is already installed (module found in %s), skipping " % (mod, modspath)
-                print_msg(msg, log, silent=testing)
-                log.info(msg)
-            else:
-                log.debug("%s is not installed yet, so retaining it" % mod)
-                easyconfigs.append(ec)
+        easyconfigs = skip_available(easyconfigs, testing=testing)
 
     # determine an order that will allow all specs in the set to build
     if len(easyconfigs) > 0:
@@ -298,8 +291,8 @@ def main(testing_data=(None, None)):
         # the options to ignore (help options can't reach here)
         ignore_opts = ['robot', 'job']
 
-        # cmd_args is in form --longopt=value
-        opts = [x for x in cmd_args if not x.split('=')[0] in ['--%s' % y for y in ignore_opts]]
+        # generate_cmd_line returns the options in form --longopt=value
+        opts = [x for x in eb_go.generate_cmd_line() if not x.split('=')[0] in ['--%s' % y for y in ignore_opts]]
 
         quoted_opts = subprocess.list2cmdline(opts)
 
@@ -428,6 +421,23 @@ def process_easyconfig(path, onlyBlocks=None, regtest_online=False, validate=Tru
 
     return easyconfigs
 
+def skip_available(easyconfigs, testing=False):
+    """Skip building easyconfigs for which a module is already available."""
+    m = Modules()
+    easyconfigs, check_easyconfigs = [], easyconfigs
+    for ec in check_easyconfigs:
+        module = ec['module']
+        mod = "%s (version %s)" % (module[0], module[1])
+        modspath = mk_module_path(curr_module_paths() + [os.path.join(config.install_path("mod"), 'all')])
+        if m.exists(module[0], module[1], modspath):
+            msg = "%s is already installed (module found in %s), skipping " % (mod, modspath)
+            print_msg(msg, log, silent=testing)
+            log.info(msg)
+        else:
+            log.debug("%s is not installed yet, so retaining it" % mod)
+            easyconfigs.append(ec)
+    return easyconfigs
+
 def resolve_dependencies(unprocessed, robot, force=False):
     """
     Work through the list of easyconfigs to determine an optimal order
@@ -612,7 +622,7 @@ def obtain_path(specs, paths, try_to_generate=False, exit_on_error=True, silent=
 
     # if no easyconfig files/paths were provided, but we did get a software name,
     # we can try and find a suitable easyconfig ourselves, or generate one if we can
-    (generated, fn) = easyconfig.obtain_ec_for(specs, paths, None)
+    (generated, fn) = easyconfig.tools.obtain_ec_for(specs, paths, None)
     if not generated:
         return (fn, generated)
     else:
@@ -636,7 +646,7 @@ def robot_find_easyconfig(path, module):
     """
     name, version = module
     # candidate easyconfig paths
-    easyconfigsPaths = easyconfig.create_paths(path, name, version)
+    easyconfigsPaths = easyconfig.tools.create_paths(path, name, version)
     for easyconfigPath in easyconfigsPaths:
         log.debug("Checking easyconfig path %s" % easyconfigPath)
         if os.path.isfile(easyconfigPath):
@@ -1059,12 +1069,15 @@ def build_easyconfigs(easyconfigs, output_dir, test_results, options):
             os.chdir(base_dir)
             modify_env(os.environ, base_env)
 
-            steps = EasyBlock.get_steps()
+            steps = EasyBlock.get_steps(iteration_count=app.det_iter_cnt())
 
-            for (step_name, _, step_methods, _) in steps:
-                for step_method in step_methods:
-                    method_name = '_'.join(step_method.func_code.co_names)
-                    perform_step('_'.join([step_name, method_name]), app, step_method, applog)
+            for (step_name, _, step_methods, skippable) in steps:
+                if skippable and step_name in app.cfg['skipsteps']:
+                    log.info("Skipping step %s" % step_name)
+                else:
+                    for step_method in step_methods:
+                        method_name = '_'.join(step_method.func_code.co_names)
+                        perform_step('_'.join([step_name, method_name]), app, step_method, applog)
 
             # close log and move it
             app.close_log()
@@ -1171,7 +1184,7 @@ def regtest(options, easyconfig_paths):
     # create base directory, which is used to place
     # all log files and the test output as xml
     basename = "easybuild-test-%s" % datetime.now().strftime("%Y%m%d%H%M%S")
-    var = config.environmentVariables['test_output_path']
+    var = config.oldstyle_environment_variables['test_output_path']
     if options.regtest_output_dir:
         output_dir = options.regtest_output_dir
     elif var in os.environ:
@@ -1201,6 +1214,10 @@ def regtest(options, easyconfig_paths):
         except EasyBuildError, err:
             test_results.append((ecfile, 'parsing_easyconfigs', 'easyconfig file error: %s' % err, log))
 
+    # skip easyconfigs for which a module is already available, unless forced
+    if not options.force:
+        easyconfigs = skip_available(easyconfigs)
+
     if options.sequential:
         return build_easyconfigs(easyconfigs, output_dir, test_results, options)
     else:
