@@ -1,7 +1,5 @@
 ##
-# Copyright 2012 Ghent University
-# Copyright 2012 Toon Willems
-# Copyright 2012 Kenneth Hoste
+# Copyright 2012-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -24,36 +22,50 @@
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
 ##
+"""
+Unit tests for easyconfig.py
+
+@author: Toon Willems (Ghent University)
+@author: Kenneth Hoste (Ghent University)
+@author: Stijn De Weirdt (Ghent University)
+"""
+
 import os
 import re
 import shutil
 import tempfile
+from vsc import fancylogger
 
 import easybuild.framework.easyconfig as easyconfig
-from unittest import TestCase, TestSuite
+from unittest import TestCase, TestSuite, main
 from easybuild.framework.easyblock import EasyBlock
-from easybuild.framework.easyconfig import EasyConfig, tweak, obtain_ec_for
+from easybuild.framework.easyconfig.easyconfig import EasyConfig, det_installversion
+from easybuild.framework.easyconfig.tools import tweak, obtain_ec_for
 from easybuild.test.utilities import find_full_path
-from easybuild.tools.build_log import EasyBuildError, get_log
+from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.systemtools import get_shared_lib_ext
 
 class EasyConfigTest(TestCase):
     """ Baseclass for easyblock testcases """
+    contents = None
 
     def setUp(self):
         """ create temporary easyconfig file """
-        self.log = get_log("EasyConfigTest")
-        self.eb_file = "/tmp/easyconfig_test_file.eb"
-        f = open(self.eb_file, "w")
-        f.write(self.contents)
-        f.close()
+        self.log = fancylogger.getLogger("EasyConfigTest", fname=False)
+        if self.contents is not None:
+            fd, self.eb_file = tempfile.mkstemp(prefix='easyconfig_test_file_', suffix='.eb')
+            os.close(fd)
+            f = open(self.eb_file, "w")
+            f.write(self.contents)
+            f.close()
         self.cwd = os.getcwd()
 
         self.all_stops = [x[0] for x in EasyBlock.get_steps()]
 
     def tearDown(self):
         """ make sure to remove the temporary file """
-        os.remove(self.eb_file)
+        if self.contents is not None:
+            os.remove(self.eb_file)
         os.chdir(self.cwd)
 
     def assertErrorRegex(self, error, regex, call, *args):
@@ -119,7 +131,7 @@ stop = 'notvalid'
     def runTest(self):
         """ test other validations beside mandatory variables """
         eb = EasyConfig(self.eb_file, validate=False, valid_stops=self.all_stops)
-        self.assertErrorRegex(EasyBuildError, "\w* provided '\w*' is not valid", eb.validate)
+        self.assertErrorRegex(EasyBuildError, r"\w* provided '\w*' is not valid", eb.validate)
 
         eb['stop'] = 'patch'
         # this should now not crash
@@ -254,7 +266,8 @@ dependencies = [('first', '1.1'), {'name': 'second', 'version': '2.2'}]
         extra_vars.extend([('mandatory_key', ['default', 'another mandatory key', easyconfig.MANDATORY])])
 
         # test extra mandatory vars
-        self.assertErrorRegex(EasyBuildError, "mandatory variables \S* not provided", EasyConfig, self.eb_file, extra_vars)
+        self.assertErrorRegex(EasyBuildError, r"mandatory variables \S* not provided",
+                              EasyConfig, self.eb_file, extra_vars)
 
         self.contents += '\nmandatory_key = "value"'
         self.setUp()
@@ -290,7 +303,8 @@ sourceURLs = ['http://google.com']
 class TestTweaking(EasyConfigTest):
     """test tweaking ability of easyconfigs"""
 
-    tweaked_fn = "/tmp/tweaked.eb"
+    fd, tweaked_fn = tempfile.mkstemp(prefix='easybuild-tweaked-', suffix='.eb')
+    os.close(fd)
 
     patches = ["t1.patch", ("t2.patch", 1), ("t3.patch", "test"), ("t4.h", "include")]
     contents = """
@@ -319,7 +333,7 @@ patches = %s
                   'toolchain_version': tcver,
                   'patches': extra_patches
                  }
-        tweak(self.eb_file, self.tweaked_fn, tweaks, self.log)
+        tweak(self.eb_file, self.tweaked_fn, tweaks)
 
         eb = EasyConfig(self.tweaked_fn, valid_stops=self.all_stops)
         self.assertEqual(eb['version'], ver)
@@ -329,8 +343,11 @@ patches = %s
         self.assertEqual(eb['patches'], extra_patches + self.patches)
 
         eb = EasyConfig(self.eb_file, valid_stops=self.all_stops)
+        # eb['toolchain']['version'] = tcver does not work as expected with templating enabled
+        eb.enable_templating = False
         eb['version'] = ver
         eb['toolchain']['version'] = tcver
+        eb.enable_templating = True
         eb.dump(self.eb_file)
 
         tweaks = {
@@ -340,7 +357,7 @@ patches = %s
                   'foo': "bar"
                  }
 
-        tweak(self.eb_file, self.tweaked_fn, tweaks, self.log)
+        tweak(self.eb_file, self.tweaked_fn, tweaks)
 
         eb = EasyConfig(self.tweaked_fn, valid_stops=self.all_stops)
         self.assertEqual(eb['toolchain']['name'], tcname)
@@ -367,11 +384,11 @@ class TestInstallVersion(EasyConfigTest):
         tcver = "4.6.3"
         dummy = "dummy"
 
-        installver = easyconfig.det_installversion(ver, tcname, tcver, verpref, versuff)
+        installver = det_installversion(ver, tcname, tcver, verpref, versuff)
 
         self.assertEqual(installver, "%s%s-%s-%s%s" % (verpref, ver, tcname, tcver, versuff))
 
-        installver = easyconfig.det_installversion(ver, dummy, tcver, verpref, versuff)
+        installver = det_installversion(ver, dummy, tcver, verpref, versuff)
 
         self.assertEqual(installver, "%s%s%s" % (verpref, ver, versuff))
 
@@ -439,11 +456,11 @@ class TestObtainEasyconfig(EasyConfigTest):
         # should crash when no suited easyconfig file (or template) is available
         specs = {'name': 'nosuchsoftware'}
         error_regexp = ".*No easyconfig files found for software %s, and no templates available. I'm all out of ideas." % specs['name']
-        self.assertErrorRegex(EasyBuildError, error_regexp, obtain_ec_for, specs, self.ec_dir, None, self.log)
+        self.assertErrorRegex(EasyBuildError, error_regexp, obtain_ec_for, specs, [self.ec_dir], None)
 
         # should find matching easyconfig file
         specs = {'name': 'foo', 'version': '1.2.3'}
-        res = obtain_ec_for(specs, self.ec_dir, None, self.log)
+        res = obtain_ec_for(specs, [self.ec_dir], None)
         self.assertEqual(res[0], False)
         self.assertEqual(res[1], os.path.join(self.ec_dir, fns[-1]))
 
@@ -457,7 +474,7 @@ class TestObtainEasyconfig(EasyConfigTest):
                       'versionsuffix': suff
                      })
         error_regexp = ".*No toolchain name specified, and more than one available: .*"
-        self.assertErrorRegex(EasyBuildError, error_regexp, obtain_ec_for, specs, self.ec_dir, None, self.log)
+        self.assertErrorRegex(EasyBuildError, error_regexp, obtain_ec_for, specs, [self.ec_dir], None)
 
         # should be able to generate an easyconfig file that slightly differs
         ver = '3.16'
@@ -467,7 +484,7 @@ class TestObtainEasyconfig(EasyConfigTest):
                       'version': ver,
                       'foo': 'bar123'
                      })
-        res = obtain_ec_for(specs, self.ec_dir, None, self.log)
+        res = obtain_ec_for(specs, [self.ec_dir], None)
         self.assertEqual(res[1], "%s-%s-%s-%s%s.eb" % (name, ver, tcname, tcver, suff))
 
         self.assertEqual(res[0], True)
@@ -478,13 +495,13 @@ class TestObtainEasyconfig(EasyConfigTest):
         self.assertEqual(ec['toolchain'], {'name': tcname, 'version': tcver})
         # can't check for key 'foo', because EasyConfig ignores parameter names it doesn't know about
         txt = open(res[1], "r").read()
-        self.assertTrue(re.search("foo = '%s'" % specs['foo'], txt))
+        self.assertTrue(re.search('foo = "%s"' % specs['foo'], txt))
         os.remove(res[1])
 
         # should pick correct version, i.e. not newer than what's specified, if a choice needs to be made
         ver = '3.14'
         specs.update({'version': ver})
-        res = obtain_ec_for(specs, self.ec_dir, None, self.log)
+        res = obtain_ec_for(specs, [self.ec_dir], None)
         self.assertEqual(res[0], True)
         ec = EasyConfig(res[1], valid_stops=self.all_stops)
         self.assertEqual(ec['version'], specs['version'])
@@ -497,7 +514,7 @@ class TestObtainEasyconfig(EasyConfigTest):
                       'version': '3.15',
                       'toolchain_version': '4.4.5',
                      })
-        res = obtain_ec_for(specs, self.ec_dir, None, self.log)
+        res = obtain_ec_for(specs, [self.ec_dir], None)
         self.assertEqual(res[0], True)
         ec = EasyConfig(res[1], valid_stops=self.all_stops)
         self.assertEqual(ec['version'], specs['version'])
@@ -515,7 +532,7 @@ class TestObtainEasyconfig(EasyConfigTest):
                       'patches': extra_patches,
                       'dependencies': deps
                      })
-        res = obtain_ec_for(specs, self.ec_dir, None, self.log)
+        res = obtain_ec_for(specs, [self.ec_dir], None)
         self.assertEqual(res[0], True)
         ec = EasyConfig(res[1], valid_stops=self.all_stops)
         self.assertEqual(ec['patches'], specs['patches'] + patches)
@@ -524,7 +541,7 @@ class TestObtainEasyconfig(EasyConfigTest):
 
         # should use supplied filename
         fn = "my.eb"
-        res = obtain_ec_for(specs, self.ec_dir, fn, self.log)
+        res = obtain_ec_for(specs, [self.ec_dir], fn)
         self.assertEqual(res[0], True)
         self.assertEqual(res[1], fn)
         os.remove(res[1])
@@ -549,7 +566,7 @@ class TestObtainEasyconfig(EasyConfigTest):
         if tpl_full_path:
             shutil.copy2(tpl_full_path, self.ec_dir)
             specs.update({'name': 'nosuchsoftware'})
-            res = obtain_ec_for(specs, self.ec_dir, None, self.log)
+            res = obtain_ec_for(specs, [self.ec_dir], None)
             self.assertEqual(res[0], True)
             ec = EasyConfig(res[1], valid_stops=self.all_stops)
             self.assertEqual(ec['name'], specs['name'])
@@ -560,9 +577,146 @@ class TestObtainEasyconfig(EasyConfigTest):
         EasyConfigTest.tearDown(self)
         shutil.rmtree(self.ec_dir)
 
+
+class TestTemplating(EasyConfigTest):
+    """test templating validations """
+
+    inp = {
+           'name':'PI',
+           'version':'3.14',
+           'namelower':'pi',
+           'cmd': 'tar xfvz %s',
+          }
+    # don't use any escaping insanity here, since it is templated itself
+    contents = """
+name = "%(name)s"
+version = "%(version)s"
+homepage = "http://google.com"
+description = "test easyconfig %%(name)s"
+toolchain = {"name":"dummy", "version": "dummy2"}
+source_urls = [(GOOGLECODE_SOURCE)]
+sources = [SOURCE_TAR_GZ, (SOURCELOWER_TAR_GZ, '%(cmd)s')]
+""" % inp
+
+    def runTest(self):
+        """ test easyconfig templating """
+        eb = EasyConfig(self.eb_file, validate=False, valid_stops=self.all_stops)
+        eb.validate()
+        eb.generate_template_values()
+
+        self.assertEqual(eb['description'], "test easyconfig PI")
+        const_dict = dict([(x[0], x[1]) for x in easyconfig.templates.TEMPLATE_CONSTANTS])
+        self.assertEqual(eb['sources'][0], const_dict['SOURCE_TAR_GZ'] % self.inp)
+        self.assertEqual(eb['sources'][1][0], const_dict['SOURCELOWER_TAR_GZ'] % self.inp)
+        self.assertEqual(eb['sources'][1][1], 'tar xfvz %s')
+        self.assertEqual(eb['source_urls'][0], const_dict['GOOGLECODE_SOURCE'] % self.inp)
+
+        # test the escaping insanity here (ie all the crap we allow in easyconfigs)
+        eb['description'] = "test easyconfig % %% %s% %%% %(name)s %%(name)s %%%(name)s %%%%(name)s"
+        self.assertEqual(eb['description'], "test easyconfig % %% %s% %%% PI %(name)s %PI %%(name)s")
+
+
+class TestTemplatingDoc(EasyConfigTest):
+    """test templating documentation"""
+    def runTest(self):
+        """test templating documentation"""
+        doc = easyconfig.templates.template_documentation()
+        # expected length: 1 per constant and 1 extra per constantgroup
+        temps = [
+                 easyconfig.templates.TEMPLATE_NAMES_EASYCONFIG,
+                 easyconfig.templates.TEMPLATE_NAMES_CONFIG,
+                 easyconfig.templates.TEMPLATE_NAMES_LOWER,
+                 easyconfig.templates.TEMPLATE_NAMES_EASYBLOCK_RUN_STEP,
+                 easyconfig.templates.TEMPLATE_CONSTANTS,
+                ]
+        self.assertEqual(len(doc.split('\n')), sum([len(temps)] + [len(x) for x in temps]))
+
+class TestConstantDoc(EasyConfigTest):
+    """test constant documentation"""
+    def runTest(self):
+        """test constant documentation"""
+        doc = easyconfig.constants.constant_documentation()
+        # expected length: 1 per constant and 1 extra per constantgroup
+        temps = [
+                 easyconfig.constants.EASYCONFIG_CONSTANTS,
+                ]
+        self.assertEqual(len(doc.split('\n')), sum([len(temps)] + [len(x) for x in temps]))
+
+class TestBuildOptions(EasyConfigTest):
+    """Test configure/build/install options, both strings and lists."""
+
+    orig_contents = """
+name = "pi"
+version = "3.14"
+homepage = "http://google.com"
+description = "test easyconfig"
+toolchain = {"name":"dummy", "version": "dummy"}
+"""
+    contents = orig_contents
+
+    def runTest(self):
+        """Test configure/build/install options, both strings and lists."""
+
+        # configopts as string
+        configopts = '--opt1 --opt2=foo'
+        self.contents = self.orig_contents + "\nconfigopts = '%s'" % configopts
+        self.setUp()
+        eb = EasyConfig(self.eb_file, valid_stops=self.all_stops)
+
+        self.assertEqual(eb['configopts'], configopts)
+
+        # configopts as list
+        configopts = ['--opt1 --opt2=foo', '--opt1 --opt2=bar']
+        self.contents = self.orig_contents + "\nconfigopts = %s" % str(configopts)
+        self.setUp()
+        eb = EasyConfig(self.eb_file, valid_stops=self.all_stops)
+
+        self.assertEqual(eb['configopts'][0], configopts[0])
+        self.assertEqual(eb['configopts'][1], configopts[1])
+
+        # also makeopts and installopts as lists
+        makeopts = ['CC=foo' ,'CC=bar']
+        installopts = ['FOO=foo' ,'BAR=bar']
+        self.contents = self.orig_contents + "\nconfigopts = %s" % str(configopts)
+        self.contents += "\nmakeopts = %s" % str(makeopts)
+        self.contents += "\ninstallopts = %s" % str(installopts)
+        self.setUp()
+        eb = EasyConfig(self.eb_file, valid_stops=self.all_stops)
+
+        self.assertEqual(eb['configopts'][0], configopts[0])
+        self.assertEqual(eb['configopts'][1], configopts[1])
+        self.assertEqual(eb['makeopts'][0], makeopts[0])
+        self.assertEqual(eb['makeopts'][1], makeopts[1])
+        self.assertEqual(eb['installopts'][0], installopts[0])
+        self.assertEqual(eb['installopts'][1], installopts[1])
+
+        # error should be thrown if lists are not equal
+        installopts = ['FOO=foo', 'BAR=bar', 'BAZ=baz']
+        self.contents = self.orig_contents + "\nconfigopts = %s" % str(configopts)
+        self.contents += "\nmakeopts = %s" % str(makeopts)
+        self.contents += "\ninstallopts = %s" % str(installopts)
+        self.setUp()
+        eb = EasyConfig(self.eb_file, valid_stops=self.all_stops, validate=False)
+        self.assertErrorRegex(EasyBuildError, "Build option lists for iterated build should have same length",
+                              eb.validate)
+
+        # list with a single element is OK, is treated as a string
+        installopts = ['FOO=foo']
+        self.contents = self.orig_contents + "\nconfigopts = %s" % str(configopts)
+        self.contents += "\nmakeopts = %s" % str(makeopts)
+        self.contents += "\ninstallopts = %s" % str(installopts)
+        self.setUp()
+        eb = EasyConfig(self.eb_file, valid_stops=self.all_stops)
+
 def suite():
     """ return all the tests in this file """
     return TestSuite([TestDependency(), TestEmpty(), TestExtraOptions(),
                       TestMandatory(), TestSharedLibExt(), TestSuggestions(),
                       TestValidation(), TestTweaking(), TestInstallVersion(),
-                      TestObtainEasyconfig()])
+                      TestObtainEasyconfig(),
+                      TestTemplating(), TestTemplatingDoc(), TestConstantDoc(),
+                      ])
+
+
+if __name__ == '__main__':
+    main()
