@@ -1,7 +1,5 @@
 ##
-# Copyright 2012 Ghent University
-# Copyright 2012 Stijn De Weirdt
-# Copyright 2012 Kenneth Hoste
+# Copyright 2012-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -26,6 +24,9 @@
 ##
 """
 Support for BLACS as toolchain linear algebra library.
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Kenneth Hoste (Ghent University)
 """
 
 from easybuild.tools.toolchain.linalg import LinAlg
@@ -33,8 +34,7 @@ from easybuild.tools.toolchain.linalg import LinAlg
 
 class Blacs(LinAlg):
     """
-    Trivial class
-        provides BLACS
+    Trivial class, provides BLACS support.
     """
     BLACS_MODULE_NAME = ['BLACS']
     BLACS_LIB = ["blacsCinit", "blacsF77init", "blacs"]
