@@ -1,7 +1,5 @@
 ##
-# Copyright 2012 Ghent University
-# Copyright 2012 Stijn De Weirdt
-# Copyright 2012 Kenneth Hoste
+# Copyright 2012-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -26,15 +24,16 @@
 ##
 """
 Support for ScaLAPACK as toolchain linear algebra library.
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Kenneth Hoste (Ghent University)
 """
 
 from easybuild.tools.toolchain.linalg import LinAlg
 
 
 class ScaLAPACK(LinAlg):
-    """Trivial class
-        provides ScaLAPACK
-    """
+    """Trivial class, provides ScaLAPACK support."""
     SCALAPACK_MODULE_NAME = ['ScaLAPACK']
     SCALAPACK_LIB = ['scalapack']
 
