@@ -1,4 +1,4 @@
-##
+# #
 # Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
@@ -21,7 +21,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 EasyBuild logger and log utilities, including our own EasybuildError class.
 
@@ -36,6 +36,9 @@ import os
 import sys
 from copy import copy
 from vsc import fancylogger
+
+from easybuild.tools.version import VERSION
+
 # EasyBuild message prefix
 EB_MSG_PREFIX = "=="
 
@@ -47,6 +50,7 @@ class EasyBuildError(Exception):
     def __init__(self, msg):
         Exception.__init__(self, msg)
         self.msg = msg
+
     def __str__(self):
         return repr(self.msg)
 
@@ -61,6 +65,7 @@ class EasyBuildLog(fancylogger.FancyLogger):
     raiseError = True
 
     def caller_info(self):
+        """Return string with caller info."""
         (filepath, line, function_name) = self.findCaller()
         filepath_dirs = filepath.split(os.path.sep)
 
@@ -71,14 +76,20 @@ class EasyBuildLog(fancylogger.FancyLogger):
                 break
         return "(at %s:%s in %s)" % (os.path.sep.join(filepath_dirs), line, function_name)
 
+    def deprecated(self, msg, max_ver):
+        """Print deprecation warning or raise an EasyBuildError, depending on max version allowed."""
+        fancylogger.FancyLogger.deprecated(self, msg, str(VERSION), max_ver, exception=EasyBuildError)
+
     def error(self, msg, *args, **kwargs):
+        """Print error message and raise an EasyBuildError."""
         newMsg = "EasyBuild crashed with an error %s: %s" % (self.caller_info(), msg)
         fancylogger.FancyLogger.error(self, newMsg, *args, **kwargs)
         if self.raiseError:
             raise EasyBuildError(newMsg)
 
     def exception(self, msg, *args):
-        # # don't raise the exception from within error
+        """Print exception message and raise EasyBuildError."""
+        # don't raise the exception from within error
         newMsg = "EasyBuild encountered an exception %s: %s" % (self.caller_info(), msg)
 
         self.raiseError = False
@@ -105,6 +116,7 @@ fancylogger.logToFile(filename=os.devnull)
 # EasyBuildLog
 _init_easybuildlog = fancylogger.getLogger(fname=False)
 
+
 def get_log(name=None):
     """
     Generate logger object
@@ -112,6 +124,7 @@ def get_log(name=None):
     # fname is always get_log, useless
     log = fancylogger.getLogger(name, fname=False)
     log.info("Logger started for %s." % name)
+    log.deprecated("get_log", "2.0")
     return log
 
 
