@@ -1,11 +1,5 @@
 ##
-# Copyright 2009-2012 Ghent University
-# Copyright 2009-2012 Stijn De Weirdt
-# Copyright 2010 Dries Verdegem
-# Copyright 2010-2012 Kenneth Hoste
-# Copyright 2011 Pieter De Baets
-# Copyright 2011-2012 Jens Timmerman
-# Copyright 2012 Toon Willems
+# Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -29,7 +23,14 @@
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
 ##
 """
-Set of file tools
+Set of file tools.
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Dries Verdegem (Ghent University)
+@author: Kenneth Hoste (Ghent University)
+@author: Pieter De Baets (Ghent University)
+@author: Jens Timmerman (Ghent University)
+@author: Toon Willems (Ghent University)
 """
 import errno
 import os
@@ -41,14 +42,14 @@ import subprocess
 import tempfile
 import time
 import urllib
+from vsc import fancylogger
 
 import easybuild.tools.environment as env
 from easybuild.tools.asyncprocess import Popen, PIPE, STDOUT
 from easybuild.tools.asyncprocess import send_all, recv_some
-from easybuild.tools.build_log import get_log
 
 
-log = get_log('fileTools')
+_log = fancylogger.getLogger('filetools', fname=False)
 errorsFoundInLog = 0
 
 # constants for strictness levels
@@ -66,24 +67,24 @@ def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
     - returns the directory name in case of success
     """
     if not os.path.isfile(fn):
-        log.error("Can't extract file %s: no such file" % fn)
+        _log.error("Can't extract file %s: no such file" % fn)
 
     if not os.path.isdir(dest):
         ## try to create it
         try:
             os.makedirs(dest)
         except OSError, err:
-            log.exception("Can't extract file %s: directory %s can't be created: %err " % (fn, dest, err))
+            _log.exception("Can't extract file %s: directory %s can't be created: %err " % (fn, dest, err))
 
     ## use absolute pathnames from now on
     absDest = os.path.abspath(dest)
 
     ## change working directory
     try:
-        log.debug("Unpacking %s in directory %s." % (fn, absDest))
+        _log.debug("Unpacking %s in directory %s." % (fn, absDest))
         os.chdir(absDest)
     except OSError, err:
-        log.error("Can't change to directory %s: %s" % (absDest, err))
+        _log.error("Can't change to directory %s: %s" % (absDest, err))
 
     if not cmd:
         cmd = extract_cmd(fn, overwrite=overwrite)
@@ -91,7 +92,7 @@ def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
         # complete command template with filename
         cmd = cmd % fn
     if not cmd:
-        log.error("Can't extract file %s with unknown filetype" % fn)
+        _log.error("Can't extract file %s with unknown filetype" % fn)
 
     if extra_options:
         cmd = "%s %s" % (cmd, extra_options)
@@ -102,7 +103,7 @@ def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
 
 def download_file(filename, url, path):
 
-    log.debug("Downloading %s from %s to %s" % (filename, url, path))
+    _log.debug("Downloading %s from %s to %s" % (filename, url, path))
 
     # make sure directory exists
     basedir = os.path.dirname(path)
@@ -118,19 +119,19 @@ def download_file(filename, url, path):
         (_, httpmsg) = urllib.urlretrieve(url, path)
 
         if httpmsg.type == "text/html" and not filename.endswith('.html'):
-            log.warning("HTML file downloaded but not expecting it, so assuming invalid download.")
-            log.debug("removing downloaded file %s from %s" % (filename, path))
+            _log.warning("HTML file downloaded but not expecting it, so assuming invalid download.")
+            _log.debug("removing downloaded file %s from %s" % (filename, path))
             try:
                 os.remove(path)
             except OSError, err:
-                log.error("Failed to remove downloaded file:" % err)
+                _log.error("Failed to remove downloaded file:" % err)
         else:
-            log.info("Downloading file %s from url %s: done" % (filename, url))
+            _log.info("Downloading file %s from url %s: done" % (filename, url))
             downloaded = True
             return path
 
         attempt_cnt += 1
-        log.warning("Downloading failed at attempt %s, retrying..." % attempt_cnt)
+        _log.warning("Downloading failed at attempt %s, retrying..." % attempt_cnt)
 
     # failed to download after multiple attempts
     return None
@@ -162,11 +163,11 @@ def find_base_dir():
         try:
             os.chdir(newDir)
         except OSError, err:
-            log.exception("Changing to dir %s from current dir %s failed: %s" % (newDir, os.getcwd(), err))
+            _log.exception("Changing to dir %s from current dir %s failed: %s" % (newDir, os.getcwd(), err))
         lst = get_local_dirs_purged()
 
-    log.debug("Last dir list %s" % lst)
-    log.debug("Possible new dir %s found" % newDir)
+    _log.debug("Last dir list %s" % lst)
+    _log.debug("Possible new dir %s found" % newDir)
     return newDir
 
 
@@ -195,6 +196,14 @@ def extract_cmd(fn, overwrite=False):
     if ff[-1] == 'tbz':
         ftype = 'tar xjf %s'
 
+    # xzipped or xzipped tarball
+    if ff[-1] == 'xz':
+        ftype = 'unxz %s'
+        if ff[-2] == 'tar':
+            ftype = 'unxz %s --stdout | tar x'
+    if ff[-1] == 'txz':
+        ftype = 'unxz %s --stdout | tar x'
+
     # tarball
     if ff[-1] == 'tar':
         ftype = 'tar xf %s'
@@ -207,7 +216,7 @@ def extract_cmd(fn, overwrite=False):
             ftype = 'unzip -qq %s'
 
     if not ftype:
-        log.error('Unknown file type from file %s (%s)' % (fn, ff))
+        _log.error('Unknown file type from file %s (%s)' % (fn, ff))
 
     return ftype % fn
 
@@ -219,25 +228,25 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
     """
 
     if not os.path.isfile(patchFile):
-        log.error("Can't find patch %s: no such file" % patchFile)
+        _log.error("Can't find patch %s: no such file" % patchFile)
         return
 
     if fn and not os.path.isfile(fn):
-        log.error("Can't patch file %s: no such file" % fn)
+        _log.error("Can't patch file %s: no such file" % fn)
         return
 
     if not os.path.isdir(dest):
-        log.error("Can't patch directory %s: no such directory" % dest)
+        _log.error("Can't patch directory %s: no such directory" % dest)
         return
 
     ## copy missing files
     if copy:
         try:
             shutil.copy2(patchFile, dest)
-            log.debug("Copied patch %s to dir %s" % (patchFile, dest))
+            _log.debug("Copied patch %s to dir %s" % (patchFile, dest))
             return 'ok'
         except IOError, err:
-            log.error("Failed to copy %s to dir %s: %s" % (patchFile, dest, err))
+            _log.error("Failed to copy %s to dir %s: %s" % (patchFile, dest, err))
             return
 
     ## use absolute paths
@@ -246,9 +255,9 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
 
     try:
         os.chdir(adest)
-        log.debug("Changing to directory %s" % adest)
+        _log.debug("Changing to directory %s" % adest)
     except OSError, err:
-        log.error("Can't change to directory %s: %s" % (adest, err))
+        _log.error("Can't change to directory %s: %s" % (adest, err))
         return
 
     if not level:
@@ -268,11 +277,11 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
                     plusLines.append(found)
             f.close()
         except IOError, err:
-            log.error("Can't read patch %s: %s" % (apatch, err))
+            _log.error("Can't read patch %s: %s" % (apatch, err))
             return
 
         if not plusLines:
-            log.error("Can't guess patchlevel from patch %s: no testfile line found in patch" % apatch)
+            _log.error("Can't guess patchlevel from patch %s: no testfile line found in patch" % apatch)
             return
 
         p = None
@@ -291,22 +300,22 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
                 p = i
                 break
             else:
-                log.debug('No match found for %s, trying next +++ line of patch file...' % f)
+                _log.debug('No match found for %s, trying next +++ line of patch file...' % f)
 
         if p == None: # p can also be zero, so don't use "not p"
             ## no match
-            log.error("Can't determine patch level for patch %s from directory %s" % (patchFile, adest))
+            _log.error("Can't determine patch level for patch %s from directory %s" % (patchFile, adest))
         else:
-            log.debug("Guessed patch level %d for patch %s" % (p, patchFile))
+            _log.debug("Guessed patch level %d for patch %s" % (p, patchFile))
 
     else:
         p = level
-        log.debug("Using specified patch level %d for patch %s" % (level, patchFile))
+        _log.debug("Using specified patch level %d for patch %s" % (level, patchFile))
 
     patchCmd = "patch -b -p%d -i %s" % (p, apatch)
     result = run_cmd(patchCmd, simple=True)
     if not result:
-        log.error("Patching with patch %s failed" % patchFile)
+        _log.error("Patching with patch %s failed" % patchFile)
         return
 
     return result
@@ -324,7 +333,7 @@ def adjust_cmd(func):
                 if os.path.exists(fp):
                     extra = ". %s &&%s" % (fp, extra)
                 else:
-                    log.warning("Can't find file %s" % fil)
+                    _log.warning("Can't find file %s" % fil)
 
             cmd = "%s %s" % (extra, cmd)
 
@@ -349,14 +358,14 @@ def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True
         if path:
             os.chdir(path)
 
-        log.debug("run_cmd: running cmd %s (in %s)" % (cmd, os.getcwd()))
+        _log.debug("run_cmd: running cmd %s (in %s)" % (cmd, os.getcwd()))
     except:
-        log.info("running cmd %s in non-existing directory, might fail!" % cmd)
+        _log.info("running cmd %s in non-existing directory, might fail!" % cmd)
 
     ## Log command output
     if log_output:
         runLog = tempfile.NamedTemporaryFile(suffix='.log', prefix='easybuild-run_cmd-')
-        log.debug('run_cmd: Command output will be logged to %s' % runLog.name)
+        _log.debug('run_cmd: Command output will be logged to %s' % runLog.name)
         runLog.write(cmd + "\n\n")
     else:
         runLog = None
@@ -367,7 +376,7 @@ def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True
         p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                            stdin=subprocess.PIPE, close_fds=True, executable="/bin/bash")
     except OSError, err:
-        log.error("run_cmd init cmd %s failed:%s" % (cmd, err))
+        _log.error("run_cmd init cmd %s failed:%s" % (cmd, err))
     if inp:
         p.stdin.write(inp)
     p.stdin.close()
@@ -416,9 +425,9 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
         if path:
             os.chdir(path)
 
-        log.debug("run_cmd_qa: running cmd %s (in %s)" % (cmd, os.getcwd()))
+        _log.debug("run_cmd_qa: running cmd %s (in %s)" % (cmd, os.getcwd()))
     except:
-        log.info("running cmd %s in non-existing directory, might fail!" % cmd)
+        _log.info("running cmd %s in non-existing directory, might fail!" % cmd)
 
     # Part 1: process the QandA dictionary
     # given initial set of Q and A (in dict), return dict of reg. exp. and A
@@ -443,14 +452,14 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
         if regQ.search(q):
             return (a, regQ)
         else:
-            log.error("runqanda: Question %s converted in %s does not match itself" % (q, regQtxt))
+            _log.error("runqanda: Question %s converted in %s does not match itself" % (q, regQtxt))
 
     newQA = {}
-    log.debug("newQA: ")
+    _log.debug("newQA: ")
     for question, answer in qa.items():
         (a, regQ) = process_QA(question, answer)
         newQA[regQ] = a
-        log.debug("newqa[%s]: %s" % (regQ.pattern, a))
+        _log.debug("newqa[%s]: %s" % (regQ.pattern, a))
 
     newstdQA = {}
     if std_qa:
@@ -459,14 +468,14 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             if not answer.endswith('\n'):
                 answer += '\n'
             newstdQA[regQ] = answer
-            log.debug("newstdQA[%s]: %s" % (regQ.pattern, answer))
+            _log.debug("newstdQA[%s]: %s" % (regQ.pattern, answer))
 
     new_no_qa = []
     if no_qa:
         # simple statements, can contain wildcards
         new_no_qa = [re.compile(r"" + x + r"[\s\n]*$") for x in no_qa]
 
-    log.debug("New noQandA list is: %s" % [x.pattern for x in new_no_qa])
+    _log.debug("New noQandA list is: %s" % [x.pattern for x in new_no_qa])
 
     # Part 2: Run the command and answer questions
     # - this needs asynchronous stdout
@@ -475,10 +484,10 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
     if log_all:
         try:
             runLog = tempfile.NamedTemporaryFile(suffix='.log', prefix='easybuild-cmdqa-')
-            log.debug('run_cmd_qa: Command output will be logged to %s' % runLog.name)
+            _log.debug('run_cmd_qa: Command output will be logged to %s' % runLog.name)
             runLog.write(cmd + "\n\n")
         except IOError, err:
-            log.error("Opening log file for Q&A failed: %s" % err)
+            _log.error("Opening log file for Q&A failed: %s" % err)
     else:
         runLog = None
 
@@ -487,7 +496,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
     try:
         p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, stdin=PIPE, close_fds=True, executable="/bin/bash")
     except OSError, err:
-        log.error("run_cmd_qa init cmd %s failed:%s" % (cmd, err))
+        _log.error("run_cmd_qa init cmd %s failed:%s" % (cmd, err))
 
     # initial short sleep
     time.sleep(0.1)
@@ -506,7 +515,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             stdoutErr += tmpOut
         # recv_some may throw Exception
         except (IOError, Exception), err:
-            log.debug("run_cmd_qa cmd %s: read failed: %s" % (cmd, err))
+            _log.debug("run_cmd_qa cmd %s: read failed: %s" % (cmd, err))
             tmpOut = None
 
         hit = False
@@ -514,7 +523,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             res = q.search(stdoutErr)
             if tmpOut and res:
                 fa = a % res.groupdict()
-                log.debug("run_cmd_qa answer %s question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
+                _log.debug("run_cmd_qa answer %s question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
                 send_all(p, fa)
                 hit = True
                 break
@@ -523,7 +532,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
                 res = q.search(stdoutErr)
                 if tmpOut and res:
                     fa = a % res.groupdict()
-                    log.debug("run_cmd_qa answer %s standard question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
+                    _log.debug("run_cmd_qa answer %s standard question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
                     send_all(p, fa)
                     hit = True
                     break
@@ -534,7 +543,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
                     noqa = False
                     for r in new_no_qa:
                         if r.search(stdoutErr):
-                            log.debug("runqanda: noQandA found for out %s" % stdoutErr[-50:])
+                            _log.debug("runqanda: noQandA found for out %s" % stdoutErr[-50:])
                             noqa = True
                     if not noqa:
                         hitCount += 1
@@ -549,9 +558,9 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
                 os.killpg(p.pid, signal.SIGKILL)
                 os.kill(p.pid, signal.SIGKILL)
             except OSError, err:
-                log.debug("run_cmd_qa exception caught when killing child process: %s" % err)
-            log.debug("run_cmd_qa: full stdouterr: %s" % stdoutErr)
-            log.error("run_cmd_qa: cmd %s : Max nohits %s reached: end of output %s" % (cmd,
+                _log.debug("run_cmd_qa exception caught when killing child process: %s" % err)
+            _log.debug("run_cmd_qa: full stdouterr: %s" % stdoutErr)
+            _log.error("run_cmd_qa: cmd %s : Max nohits %s reached: end of output %s" % (cmd,
                                                                                     maxHitCount,
                                                                                     stdoutErr[-500:]
                                                                                     ))
@@ -567,7 +576,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             if runLog:
                 runLog.write(readTxt)
     except IOError, err:
-        log.debug("runqanda cmd %s: remaining data read failed: %s" % (cmd, err))
+        _log.debug("runqanda cmd %s: remaining data read failed: %s" % (cmd, err))
 
     # Not needed anymore. Subprocess does this correct?
     # ec=os.WEXITSTATUS(ec)
@@ -589,7 +598,7 @@ def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
         check_ec = True
         use_regexp = True
     else:
-        log.error("invalid strictness setting: %s" % strictness)
+        _log.error("invalid strictness setting: %s" % strictness)
 
     # allow for overriding the regexp setting
     if not regexp:
@@ -598,15 +607,15 @@ def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
     if ec and (log_all or log_ok):
         # We don't want to error if the user doesn't care
         if check_ec:
-            log.error('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
+            _log.error('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
         else:
-            log.warn('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
+            _log.warn('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
 
     if not ec:
         if log_all:
-            log.info('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
+            _log.info('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
         else:
-            log.debug('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
+            _log.debug('cmd "%s" exited with exitcode %s and output:\n%s' % (cmd, ec, stdouterr))
 
     # parse the stdout/stderr for errors when strictness dictates this or when regexp is passed in
     if use_regexp or regexp:
@@ -614,9 +623,9 @@ def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
         if len(res) > 0:
             message = "Found %s errors in command output (output: %s)" % (len(res), ", ".join([r[0] for r in res]))
             if use_regexp:
-                log.error(message)
+                _log.error(message)
             else:
-                log.warn(message)
+                _log.warn(message)
 
     if simple:
         if ec:
@@ -640,15 +649,15 @@ def modify_env(old, new):
         if key in oldKeys:
             ## hmm, smart checking with debug logging
             if not new[key] == old[key]:
-                log.debug("Key in new environment found that is different from old one: %s (%s)" % (key, new[key]))
+                _log.debug("Key in new environment found that is different from old one: %s (%s)" % (key, new[key]))
                 env.setvar(key, new[key])
         else:
-            log.debug("Key in new environment found that is not in old one: %s (%s)" % (key, new[key]))
+            _log.debug("Key in new environment found that is not in old one: %s (%s)" % (key, new[key]))
             env.setvar(key, new[key])
 
     for key in oldKeys:
         if not key in newKeys:
-            log.debug("Key in old environment found that is not in new one: %s (%s)" % (key, old[key]))
+            _log.debug("Key in old environment found that is not in new one: %s (%s)" % (key, old[key]))
             os.unsetenv(key)
             del os.environ[key]
 
@@ -682,11 +691,11 @@ def parse_log_for_error(txt, regExp=None, stdout=True, msg=None):
 
     if regExp and type(regExp) == bool:
         regExp = r"(?<![(,]|\w)(?:error|segmentation fault|failed)(?![(,]|\.?\w)"
-        log.debug('Using default regular expression: %s' % regExp)
+        _log.debug('Using default regular expression: %s' % regExp)
     elif type(regExp) == str:
         pass
     else:
-        log.error("parselogForError no valid regExp used: %s" % regExp)
+        _log.error("parselogForError no valid regExp used: %s" % regExp)
 
     reg = re.compile(regExp, re.I)
 
@@ -699,8 +708,8 @@ def parse_log_for_error(txt, regExp=None, stdout=True, msg=None):
 
     if stdout and res:
         if msg:
-            log.info("parseLogError msg: %s" % msg)
-        log.info("parseLogError (some may be harmless) regExp %s found:\n%s" % (regExp,
+            _log.info("parseLogError msg: %s" % msg)
+        _log.info("parseLogError (some may be harmless) regExp %s found:\n%s" % (regExp,
                                                                               '\n'.join([x[0] for x in res])
                                                                               ))
 
@@ -717,7 +726,7 @@ def adjust_permissions(name, permissionBits, add=True, onlyfiles=False, onlydirs
     name = os.path.abspath(name)
 
     if recursive:
-        log.info("Adjusting permissions recursively for %s" % name)
+        _log.info("Adjusting permissions recursively for %s" % name)
         allpaths = [name]
         for root, dirs, files in os.walk(name):
             paths = []
@@ -730,10 +739,11 @@ def adjust_permissions(name, permissionBits, add=True, onlyfiles=False, onlydirs
                 allpaths.append(os.path.join(root, path))
 
     else:
-        log.info("Adjusting permissions for %s" % name)
+        _log.info("Adjusting permissions for %s" % name)
         allpaths = [name]
 
     failed_paths = []
+    fail_cnt = 0
     for path in allpaths:
 
         try:
@@ -757,12 +767,23 @@ def adjust_permissions(name, permissionBits, add=True, onlyfiles=False, onlydirs
         except OSError, err:
             if ignore_errors:
                 # ignore errors while adjusting permissions (for example caused by bad links)
-                log.info("Failed to chmod/chown %s (but ignoring it): %s" % (path, err))
+                _log.info("Failed to chmod/chown %s (but ignoring it): %s" % (path, err))
+                fail_cnt += 1
             else:
                 failed_paths.append(path)
 
     if failed_paths:
-        log.exception("Failed to chmod/chown several paths: %s (last error: %s)" % (failed_paths, err))
+        _log.error("Failed to chmod/chown several paths: %s (last error: %s)" % (failed_paths, err))
+
+    # we ignore some errors, but if there are to many, something is definitely wrong
+    fail_ratio = fail_cnt / float(len(allpaths))
+    max_fail_ratio = 0.5
+    if fail_ratio > max_fail_ratio:
+        _log.error("%.2f%% of permissions/owner operations failed (more than %.2f%%), something must be wrong..." % \
+                  (100*fail_ratio, 100*max_fail_ratio))
+    elif fail_cnt > 0:
+        _log.debug("%.2f%% of permissions/owner operations failed, ignoring that..." % (100*fail_ratio))
+
 
 def patch_perl_script_autoflush(path):
     # patch Perl script to enable autoflush,
@@ -784,13 +805,12 @@ def patch_perl_script_autoflush(path):
         f.close()
 
     except IOError, err:
-        log.error("Failed to patch Perl configure script: %s" % err)
+        _log.error("Failed to patch Perl configure script: %s" % err)
 
 def mkdir(directory, parents=False):
     """
     Create a directory
     Directory is the path to create
-    log is the logger to which to log debugging or error info.
     
     When parents is True then no error if directory already exists
     and make parent directories as needed (cfr. mkdir -p)
@@ -798,21 +818,21 @@ def mkdir(directory, parents=False):
     if parents:
         try:
             os.makedirs(directory)
-            log.debug("Succesfully created directory %s and needed parents" % directory)
+            _log.debug("Succesfully created directory %s and needed parents" % directory)
         except OSError, err:
             if err.errno == errno.EEXIST:
-                log.debug("Directory %s already exitst" % directory)
+                _log.debug("Directory %s already exitst" % directory)
             else:
-                log.error("Failed to create directory %s: %s" % (directory, err))
+                _log.error("Failed to create directory %s: %s" % (directory, err))
     else:#not parrents
         try:
             os.mkdir(directory)
-            log.debug("Succesfully created directory %s" % directory)
+            _log.debug("Succesfully created directory %s" % directory)
         except OSError, err:
             if err.errno == errno.EEXIST:
-                log.warning("Directory %s already exitst" % directory)
+                _log.warning("Directory %s already exitst" % directory)
             else:
-                log.error("Failed to create directory %s: %s" % (directory, err))
+                _log.error("Failed to create directory %s: %s" % (directory, err))
 
 def rmtree2(path, n=3):
     """Wrapper around shutil.rmtree to make it more robust when used on NFS mounted file systems."""
@@ -824,12 +844,12 @@ def rmtree2(path, n=3):
             ok = True
             break
         except OSError, err:
-            log.debug("Failed to remove path %s with shutil.rmtree at attempt %d: %s" % (path, n, err))
+            _log.debug("Failed to remove path %s with shutil.rmtree at attempt %d: %s" % (path, n, err))
             time.sleep(2)
     if not ok:
-        log.error("Failed to remove path %s with shutil.rmtree, even after %d attempts." % (path, n))
+        _log.error("Failed to remove path %s with shutil.rmtree, even after %d attempts." % (path, n))
     else:
-        log.info("Path %s successfully removed." % path)
+        _log.info("Path %s successfully removed." % path)
 
 def copytree(src, dst, symlinks=False, ignore=None):
     """
@@ -873,7 +893,7 @@ def copytree(src, dst, symlinks=False, ignore=None):
         ignored_names = ignore(src, names)
     else:
         ignored_names = set()
-    log.debug("copytree: skipping copy of %s" % ignored_names)
+    _log.debug("copytree: skipping copy of %s" % ignored_names)
     os.makedirs(dst)
     errors = []
     for name in names:
