@@ -1,10 +1,5 @@
-##
-# Copyright 2009-2012 Ghent University
-# Copyright 2009-2012 Stijn De Weirdt
-# Copyright 2010 Dries Verdegem
-# Copyright 2010-2012 Kenneth Hoste
-# Copyright 2011 Pieter De Baets
-# Copyright 2011-2012 Jens Timmerman
+# #
+# Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -26,21 +21,26 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 Generating module files.
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Dries Verdegem (Ghent University)
+@author: Kenneth Hoste (Ghent University)
+@author: Pieter De Baets (Ghent University)
+@author: Jens Timmerman (Ghent University)
+@author: Fotis Georgatos (Uni.Lu)
 """
 import os
-import shutil
 import tempfile
+from vsc import fancylogger
 
-from easybuild.tools.build_log import get_log
 from easybuild.tools.config import install_path
-from easybuild.tools.filetools import rmtree2
-from easybuild.tools.modules import Modules
+from easybuild.tools.utilities import quote_str
 
 
-log = get_log('moduleGenerator')
+_log = fancylogger.getLogger('moduleGenerator', fname=False)
 
 # general module class
 GENERAL_CLASS = 'all'
@@ -65,7 +65,7 @@ class ModuleGenerator(object):
         # Fake mode: set installpath to temporary dir
         if self.fake:
             self.tmpdir = tempfile.mkdtemp()
-            log.debug("Fake mode: using %s (instead of %s)" % (self.tmpdir, module_path))
+            _log.debug("Fake mode: using %s (instead of %s)" % (self.tmpdir, module_path))
             module_path = self.tmpdir
 
         # Real file goes in 'all' category
@@ -81,7 +81,7 @@ class ModuleGenerator(object):
                 try:
                     os.makedirs(directory)
                 except OSError, err:
-                    log.exception("Couldn't make directory %s: %s" % (directory, err))
+                    _log.exception("Couldn't make directory %s: %s" % (directory, err))
 
         # Make a symlink from classpathFile to self.filename
         try:
@@ -93,7 +93,7 @@ class ModuleGenerator(object):
                 os.remove(self.filename)
             os.symlink(self.filename, classPathFile)
         except OSError, err:
-            log.exception("Failed to create symlink from %s to %s: %s" % (classPathFile, self.filename, err))
+            _log.exception("Failed to create symlink from %s to %s: %s" % (classPathFile, self.filename, err))
 
         return os.path.join(module_path, GENERAL_CLASS)
 
@@ -153,20 +153,20 @@ if { ![is-loaded %(name)s/%(version)s] } {
 }
 """ % {'name': name, 'version': version}
 
-    def prepend_paths(self, key, paths):
+    def prepend_paths(self, key, paths, allow_abs=False):
         """
         Generate prepend-path statements for the given list of paths.
         """
         template = "prepend-path\t%s\t\t$root/%s\n"  # $root = installdir
 
         if isinstance(paths, basestring):
-            log.info("Wrapping %s into a list before using it to prepend path %s" % (paths, key))
+            _log.info("Wrapping %s into a list before using it to prepend path %s" % (paths, key))
             paths = [paths]
 
         # make sure only relative paths are passed
         for path in paths:
-            if path.startswith(os.path.sep):
-                log.error("Absolute path %s passed to prepend_paths which only expects relative paths." % path)
+            if path.startswith(os.path.sep) and not allow_abs:
+                _log.error("Absolute path %s passed to prepend_paths which only expects relative paths." % path)
 
         statements = [template % (key, p) for p in paths]
         return ''.join(statements)
@@ -175,19 +175,6 @@ if { ![is-loaded %(name)s/%(version)s] } {
         """
         Generate setenv statement for the given key/value pair.
         """
-        return "setenv\t%s\t\t%s\n" % (key, value)
+        # quotes are needed, to ensure smooth working of EBDEVEL* modulefiles
+        return 'setenv\t%s\t\t%s\n' % (key, quote_str(value))
 
-    def __del__(self):
-        """
-        Desconstructor: clean up temporary directory used for fake modules, if any.
-        """
-        if self.fake:
-            try:
-                m = Modules([os.path.join(self.tmpdir, GENERAL_CLASS)])
-                m.add_module([[self.app.name, self.app.get_installversion()]])
-                m.unload()
-                if os.path.exists(self.tmpdir):
-                    log.info("Cleaning up fake modules dir %s" % self.tmpdir)
-                    rmtree2(self.tmpdir)
-            except OSError, err:
-                log.exception("Cleaning up fake module dir failed: %s" % err)
