@@ -1,10 +1,5 @@
 ##
-# Copyright 2009-2012 Ghent University
-# Copyright 2009-2012 Stijn De Weirdt
-# Copyright 2010 Dries Verdegem
-# Copyright 2010-2012 Kenneth Hoste
-# Copyright 2011 Pieter De Baets
-# Copyright 2011-2012 Jens Timmerman
+# Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -26,28 +21,52 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
-Modules functionality: loading modules, checking for available modules, ...
+This python module implements the environment modules functionality:
+ - loading modules
+ - checking for available modules
+ - ...
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Dries Verdegem (Ghent University)
+@author: Kenneth Hoste (Ghent University)
+@author: Pieter De Baets (Ghent University)
+@author: Jens Timmerman (Ghent University)
 """
 import os
 import re
 import subprocess
 import sys
+from vsc import fancylogger
 
-from easybuild.tools.build_log import get_log, EasyBuildError
+from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.filetools import convert_name, run_cmd
+from vsc.utils.missing import nub
 
+# software root/version environment variable name prefixes
+ROOT_ENV_VAR_NAME_PREFIX = "EBROOT"
+VERSION_ENV_VAR_NAME_PREFIX = "EBVERSION"
+DEVEL_ENV_VAR_NAME_PREFIX = "EBDEVEL"
+
+# keep track of original LD_LIBRARY_PATH, because we can change it by loading modules and break modulecmd
+# see e.g., https://bugzilla.redhat.com/show_bug.cgi?id=719785
+LD_LIBRARY_PATH = os.getenv('LD_LIBRARY_PATH', '')
 
 outputMatchers = {
     # matches whitespace and module-listing headers
     'whitespace': re.compile(r"^\s*$|^(-+).*(-+)$"),
     # matches errors such as "cmdTrace.c(713):ERROR:104: 'asdfasdf' is an unrecognized subcommand"
     'error': re.compile(r"^\S+:(?P<level>\w+):(?P<code>\d+):\s+(?P<msg>.*)$"),
-    # matches modules such as "... ictce/3.2.1.015.u4(default) ..."
-    'available': re.compile(r"\b(?P<name>\S+?)/(?P<version>[^\(\s]+)(?P<default>\(default\))?(?:\s|$)")
+    # available with --terse has one module per line
+    # matches modules such as "ictce/3.2.1.015.u4(default)"
+    # line ending with : is ignored (the modulepath in --terse)
+    'available': re.compile(r"^\s*(?P<name>\S+?)/(?P<version>[^\(\s:]+)(?P<default>\(default\))?\s*[^:\S]*$")
 }
 
+_log = fancylogger.getLogger('modules', fname=False)
+
+
 class Modules(object):
     """
     Interact with modules.
@@ -58,7 +77,7 @@ class Modules(object):
         @param modulePath: A list of paths where the modules can be located
         @type modulePath: list
         """
-        self.log = get_log(self.__class__.__name__)
+        self.log = fancylogger.getLogger(self.__class__.__name__, fname=False)
         # make sure we don't have the same path twice
         if modulePath:
             self.modulePath = set(modulePath)
@@ -83,7 +102,7 @@ class Modules(object):
         if not 'MODULEPATH' in os.environ:
             errormsg = 'MODULEPATH not found in environment'
             # check if environment-modules is found
-            module_regexp = re.compile("^module is a function\s*\nmodule\s*()")
+            module_regexp = re.compile(r"^module is a function\s*\nmodule\s*()")
             cmd = "type module"
             (out, ec) = run_cmd(cmd, log_all=False, log_ok=False)
             if ec != 0 or not module_regexp.match(out):
@@ -92,10 +111,10 @@ class Modules(object):
             self.log.error(errormsg)
 
         if self.modulePath:
-            ## set the module path environment accordingly
+            # set the module path environment accordingly
             os.environ['MODULEPATH'] = ":".join(self.modulePath)
         else:
-            ## take module path from environment
+            # take module path from environment
             self.modulePath = os.environ['MODULEPATH'].split(':')
 
         if not 'LOADEDMODULES' in os.environ:
@@ -105,8 +124,10 @@ class Modules(object):
         """
         Return list of available modules.
         """
-        if not name: name = ''
-        if not version: version = ''
+        if not name:
+            name = ''
+        if not version:
+            version = ''
 
         txt = name
         if version:
@@ -114,13 +135,13 @@ class Modules(object):
 
         modules = self.run_module('available', txt, modulePath=modulePath)
 
-        ## sort the answers in [name, version] pairs
-        ## alphabetical order, default last
+        # sort the answers in [name, version] pairs
+        # alphabetical order, default last
         modules.sort(key=lambda m: (m['name'] + (m['default'] or ''), m['version']))
         ans = [(mod['name'], mod['version']) for mod in modules]
 
         self.log.debug("module available name '%s' version '%s' in %s gave %d answers: %s" %
-            (name, version, modulePath, len(ans), ans))
+                       (name, version, modulePath, len(ans), ans))
         return ans
 
     def exists(self, name, version, modulePath=None):
@@ -140,7 +161,7 @@ class Modules(object):
                 (name, version) = mod.split('/')
             elif type(mod) == dict:
                 name = mod['name']
-                ## deal with toolchain dependency calls
+                # deal with toolchain dependency calls
                 if 'tc' in mod:
                     version = mod['tc']
                 else:
@@ -150,11 +171,11 @@ class Modules(object):
 
             mods = self.available(name, version)
             if (name, version) in mods:
-                ## ok
+                # ok
                 self.modules.append((name, version))
             else:
                 if len(mods) == 0:
-                    self.log.warning('No module %s available' % mod)
+                    self.log.warning('No module %s available' % str(mod))
                 else:
                     self.log.warning('More then one module found for %s: %s' % (mod, mods))
                 continue
@@ -204,37 +225,57 @@ class Modules(object):
         """
         Run module command.
         """
-        if type(args[0]) == list:
+        if isinstance(args[0], (list, tuple,)):
             args = args[0]
         else:
             args = list(args)
 
+        if args[0] in ('available', 'list',):
+            args.insert(0, '--terse')  # run these in terse mode for better machinereading
+
         originalModulePath = os.environ['MODULEPATH']
         if kwargs.get('modulePath', None):
             os.environ['MODULEPATH'] = kwargs.get('modulePath')
         self.log.debug('Current MODULEPATH: %s' % os.environ['MODULEPATH'])
-
         self.log.debug("Running 'modulecmd python %s' from %s..." % (' '.join(args), os.getcwd()))
+        # change our LD_LIBRARY_PATH here
+        environ = os.environ.copy()
+        environ['LD_LIBRARY_PATH'] = LD_LIBRARY_PATH
+        self.log.debug("Adjusted LD_LIBRARY_PATH from '%s' to '%s'" %
+                       (os.environ.get('LD_LIBRARY_PATH', ''), environ['LD_LIBRARY_PATH']))
+        # modulecmd is now getting an outdated LD_LIBRARY_PATH, which will be adjusted on loading a module
+        # this needs to be taken into account when updating the environment via produced output, see below
         proc = subprocess.Popen(['modulecmd', 'python'] + args,
-                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                                stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=environ)
         # stdout will contain python code (to change environment etc)
         # stderr will contain text (just like the normal module command)
         (stdout, stderr) = proc.communicate()
         os.environ['MODULEPATH'] = originalModulePath
 
         if kwargs.get('return_output', False):
-            return (stdout + stderr)
-
+            return stdout + stderr
         else:
+            # keep track of current LD_LIBRARY_PATH, so we can correct the adjusted LD_LIBRARY_PATH below
+            prev_ld_library_path = os.environ.get('LD_LIBRARY_PATH', '').split(':')[::-1]
+
             # Change the environment
             try:
                 exec stdout
             except Exception, err:
-                raise EasyBuildError("Changing environment as dictated by module failed: %s" % err)
+                raise EasyBuildError("Changing environment as dictated by module failed: %s (%s)" % (err, stdout))
+
+            # correct LD_LIBRARY_PATH as yielded by the adjustments made
+            # make sure we get the order right (reverse lists with [::-1])
+            curr_ld_library_path = os.environ.get('LD_LIBRARY_PATH', '').split(':')
+            new_ld_library_path = [x for x in nub(prev_ld_library_path + curr_ld_library_path[::-1]) if len(x)][::-1]
+
+            self.log.debug("Correcting paths in LD_LIBRARY_PATH from %s to %s" %
+                           (curr_ld_library_path, new_ld_library_path))
+            os.environ['LD_LIBRARY_PATH'] = ':'.join(new_ld_library_path)
 
             # Process stderr
             result = []
-            for line in stderr.split('\n'):  #IGNORE:E1103
+            for line in stderr.split('\n'):  # IGNORE:E1103
                 if outputMatchers['whitespace'].search(line):
                     continue
 
@@ -289,10 +330,7 @@ class Modules(object):
                 # length after splitting is 0, so empty module name?
                 self.log.error("Module with empty name loaded? ('%s')" % mod)
 
-            loaded_modules.append({
-                                   'name': mod_name,
-                                   'version': mod_version
-                                   })
+            loaded_modules.append({'name': mod_name, 'version': mod_version})
 
         return loaded_modules
 
@@ -311,12 +349,12 @@ class Modules(object):
         except IOError, err:
             self.log.error("Failed to read module file %s to determine toolchain dependencies: %s" % (modfilepath, err))
 
-        loadregex = re.compile("^\s+module load\s+(.*)$", re.M)
+        loadregex = re.compile(r"^\s+module load\s+(.*)$", re.M)
         mods = [mod.split('/') for mod in loadregex.findall(modtxt)]
 
         if depth > 0:
             # recursively determine dependencies for these dependency modules, until depth is non-positive
-            moddeps = [self.dependencies_for(modname, modversion, depth=depth-1) for (modname, modversion) in mods]
+            moddeps = [self.dependencies_for(modname, modversion, depth=depth - 1) for (modname, modversion) in mods]
         else:
             # ignore any deeper dependencies
             moddeps = []
@@ -347,7 +385,7 @@ def search_module(path, query):
 
         # TODO: get directories to ignore from  easybuild.tools.repository ?
         # remove all hidden directories?:
-        #dirnames[:] = [d for d in dirnames if not d.startswith('.')]
+        # dirnames[:] = [d for d in dirnames if not d.startswith('.')]
         try:
             dirnames.remove('.svn')
         except ValueError:
@@ -358,19 +396,28 @@ def search_module(path, query):
         except ValueError:
             pass
 
+
+def get_software_root_env_var_name(name):
+    """Return name of environment variable for software root."""
+    newname = convert_name(name, upper=True)
+    return ''.join([ROOT_ENV_VAR_NAME_PREFIX, newname])
+
+
 def get_software_root(name, with_env_var=False):
     """
     Return the software root set for a particular software name.
     """
-    name = convert_name(name, upper=True)
-    environment_key = "EBROOT%s" % name
-    legacy_key = "SOFTROOT%s" % name
+    environment_key = get_software_root_env_var_name(name)
+    newname = convert_name(name, upper=True)
+    legacy_key = "SOFTROOT%s" % newname
 
     # keep on supporting legacy installations
     if environment_key in os.environ:
         env_var = environment_key
     else:
         env_var = legacy_key
+        if legacy_key in os.environ:
+            _log.deprecated("Legacy env var %s is being relied on!" % legacy_key, "2.0")
 
     root = os.getenv(env_var)
 
@@ -379,26 +426,37 @@ def get_software_root(name, with_env_var=False):
     else:
         return root
 
+
+def get_software_version_env_var_name(name):
+    """Return name of environment variable for software root."""
+    newname = convert_name(name, upper=True)
+    return ''.join([VERSION_ENV_VAR_NAME_PREFIX, newname])
+
+
 def get_software_version(name):
     """
     Return the software version set for a particular software name.
     """
-    name = convert_name(name, upper=True)
-    environment_key = "EBVERSION%s" % name
-    legacy_key = "SOFTVERSION%s" % name
+    environment_key = get_software_version_env_var_name(name)
+    newname = convert_name(name, upper=True)
+    legacy_key = "SOFTVERSION%s" % newname
 
     # keep on supporting legacy installations
     if environment_key in os.environ:
         return os.getenv(environment_key)
     else:
+        if legacy_key in os.environ:
+            _log.deprecated("Legacy env var %s is being relied on!" % legacy_key, "2.0")
         return os.getenv(legacy_key)
 
+
 def curr_module_paths():
     """
     Return a list of current module paths.
     """
     return os.environ['MODULEPATH'].split(':')
 
+
 def mk_module_path(paths):
     """
     Create a string representing the list of module paths.
