@@ -1,11 +1,5 @@
-##
-# Copyright 2009-2012 Ghent University
-# Copyright 2009-2012 Stijn De Weirdt
-# Copyright 2010 Dries Verdegem
-# Copyright 2010-2012 Kenneth Hoste
-# Copyright 2011 Pieter De Baets
-# Copyright 2011-2012 Jens Timmerman
-# Copyright 2012 Toon Willems
+# #
+# Copyright 2009-2013 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -27,20 +21,33 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
-##
+# #
 """
 Set of repository tools
 
 We have a plain filesystem, an svn and a git repository
+
+@author: Stijn De Weirdt (Ghent University)
+@author: Dries Verdegem (Ghent University)
+@author: Kenneth Hoste (Ghent University)
+@author: Pieter De Baets (Ghent University)
+@author: Jens Timmerman (Ghent University)
+@author: Toon Willems (Ghent University)
 """
 import getpass
 import os
-import shutil
 import socket
 import tempfile
 import time
+from vsc import fancylogger
+from vsc.utils.missing import get_subclasses
 
+from easybuild.framework.easyconfig.easyconfig import EasyConfig
+from easybuild.framework.easyconfig.tools import stats_to_str
 from easybuild.tools.filetools import rmtree2
+from easybuild.tools.version import VERBOSE_VERSION
+
+_log = fancylogger.getLogger('repository', fname=False)
 
 # optional Python packages, these might be missing
 # failing imports are just ignored
@@ -50,33 +57,37 @@ from easybuild.tools.filetools import rmtree2
 try:
     import git
     from git import GitCommandError
+    HAVE_GIT = True
 except ImportError:
-    pass
+    _log.debug('Failed to import git module')
+    HAVE_GIT = False
 
 # PySVN
 try:
-    import pysvn  #@UnusedImport
-    from pysvn import ClientError #IGNORE:E0611 pysvn fails to recognize ClientError is available
+    import pysvn  # @UnusedImport
+    from pysvn import ClientError  # IGNORE:E0611 pysvn fails to recognize ClientError is available
+    HAVE_PYSVN = True
 except ImportError:
-    pass
+    _log.debug('Failed to import pysvn module')
+    HAVE_PYSVN = False
 
-from easybuild.framework.easyconfig import EasyConfig, stats_to_str
-from easybuild.tools.build_log import get_log
-from easybuild.tools.version import VERBOSE_VERSION
-
-
-log = get_log('repo')
 
 class Repository(object):
     """
     Interface for repositories
     """
+
+    DESCRIPTION = None
+
+    USABLE = True  # can the Repository be used?
+
     def __init__(self, repo_path, subdir=''):
         """
         Initialize a repository. self.repo and self.subdir will be set.
         self.wc will be set to None.
         Then, setupRepo and createWorkingCopy will be called (in that order)
         """
+        self.log = fancylogger.getLogger(self.__class__.__name__, fname=False)
         self.subdir = subdir
         self.repo = repo_path
         self.wc = None
@@ -129,6 +140,10 @@ class Repository(object):
 class FileRepository(Repository):
     """Class for file repositories."""
 
+    DESCRIPTION = ("A plain flat file repository. "
+                   "The 1st argument contains the directory where the files are stored. "
+                   "The optional 2nd argument is a subdir in that path.")
+
     def setup_repo(self):
         """
         for file based repos this will create the repo directory
@@ -160,7 +175,7 @@ class FileRepository(Repository):
         if not os.path.isdir(full_path):
             os.makedirs(full_path)
 
-        ## destination
+        # destination
         dest = os.path.join(full_path, "%s.eb" % version)
 
         try:
@@ -176,15 +191,15 @@ class FileRepository(Repository):
                 statsprefix = "\n# Build statistics\nbuildstats = ["
                 statssuffix = "]\n"
             else:
-                #statstemplate = "\nbuildstats.append(%s)\n"
+                # statstemplate = "\nbuildstats.append(%s)\n"
                 statsprefix = "\nbuildstats.append("
                 statssuffix = ")\n"
 
-            dest_file.write(statsprefix + stats_to_str(stats, log) + statssuffix)
+            dest_file.write(statsprefix + stats_to_str(stats) + statssuffix)
             dest_file.close()
 
         except IOError, err:
-            log.exception("Copying file %s to %s (wc: %s) failed (%s)" % (cfg, dest, self.wc, err))
+            self.log.exception("Copying file %s to %s (wc: %s) failed (%s)" % (cfg, dest, self.wc, err))
 
         return dest
 
@@ -194,12 +209,12 @@ class FileRepository(Repository):
         """
         full_path = os.path.join(self.wc, self.subdir, name)
         if not os.path.isdir(full_path):
-            log.debug("module (%s) has not been found in the repo" % name)
+            self.log.debug("module (%s) has not been found in the repo" % name)
             return []
 
         dest = os.path.join(full_path, "%s.eb" % version)
         if not os.path.isfile(dest):
-            log.debug("version (%s) of module (%s) has not been found in the repo" % (version, name))
+            self.log.debug("version (%s) of module (%s) has not been found in the repo" % (version, name))
             return []
 
         eb = EasyConfig(dest, validate=False)
@@ -210,6 +225,11 @@ class GitRepository(FileRepository):
     """
     Class for git repositories.
     """
+    DESCRIPTION = ("A non-empty bare git repository (created with 'git init --bare' or 'git clone --bare'). "
+                   "The 1st argumentcontains the git repository location, which can be a directory or an URL. "
+                   "The second arhument  is a path inside the repository where to save the files.")
+
+    USABLE = HAVE_GIT
 
     def __init__(self, *args):
         """
@@ -226,7 +246,8 @@ class GitRepository(FileRepository):
         try:
             git.GitCommandError
         except NameError, err:
-            log.exception("It seems like GitPython is not available: %s" % err)
+            self.log.exception("It seems like GitPython is not available: %s" % err)
+
         self.wc = tempfile.mkdtemp(prefix='git-wc-')
 
     def create_working_copy(self):
@@ -235,61 +256,67 @@ class GitRepository(FileRepository):
         """
 
         reponame = 'UNKNOWN'
-        ## try to get a copy of
+        # try to get a copy of
         try:
             client = git.Git(self.wc)
             out = client.clone(self.repo)
             # out  = 'Cloning into easybuild...'
             reponame = out.split("\n")[0].split()[-1].strip(".").strip("'")
-            log.debug("rep name is %s" % reponame)
+            self.log.debug("rep name is %s" % reponame)
         except git.GitCommandError, err:
             # it might already have existed
-            log.warning("Git local repo initialization failed, it might already exist: %s" % err)
+            self.log.warning("Git local repo initialization failed, it might already exist: %s" % err)
 
         # local repo should now exist, let's connect to it again
         try:
             self.wc = os.path.join(self.wc, reponame)
-            log.debug("connectiong to git repo in %s" % self.wc)
+            self.log.debug("connectiong to git repo in %s" % self.wc)
             self.client = git.Git(self.wc)
         except (git.GitCommandError, OSError), err:
-            log.error("Could not create a local git repo in wc %s: %s" % (self.wc, err))
+            self.log.error("Could not create a local git repo in wc %s: %s" % (self.wc, err))
 
         # try to get the remote data in the local repo
         try:
             res = self.client.pull()
-            log.debug("pulled succesfully to %s in %s" % (res, self.wc))
+            self.log.debug("pulled succesfully to %s in %s" % (res, self.wc))
         except (git.GitCommandError, OSError), err:
-            log.exception("pull in working copy %s went wrong: %s" % (self.wc, err))
+            self.log.exception("pull in working copy %s went wrong: %s" % (self.wc, err))
 
     def add_easyconfig(self, cfg, name, version, stats, append):
         """
         Add easyconfig to git repository.
         """
         dest = FileRepository.add_easyconfig(self, cfg, name, version, stats, append)
-        ## add it to version control
+        # add it to version control
         if dest:
             try:
                 self.client.add(dest)
             except GitCommandError, err:
-                log.warning("adding %s to git failed: %s" % (dest, err))
+                self.log.warning("adding %s to git failed: %s" % (dest, err))
 
     def commit(self, msg=None):
         """
         Commit working copy to git repository
         """
-        log.debug("committing in git: %s" % msg)
-        completemsg = "EasyBuild-commit from %s (time: %s, user: %s) \n%s" % (socket.gethostname(), time.strftime("%Y-%m-%d_%H-%M-%S"), getpass.getuser(), msg)
-        log.debug("git status: %s" % self.client.status())
+        self.log.debug("committing in git: %s" % msg)
+        completemsg = "EasyBuild-commit from %s (time: %s, user: %s) \n%s" % (socket.gethostname(),
+                                                                              time.strftime("%Y-%m-%d_%H-%M-%S"),
+                                                                              getpass.getuser(),
+                                                                              msg)
+        self.log.debug("git status: %s" % self.client.status())
         try:
             self.client.commit('-am "%s"' % completemsg)
-            log.debug("succesfull commit")
+            self.log.debug("succesfull commit")
         except GitCommandError, err:
-            log.warning("Commit from working copy %s (msg: %s) failed, empty commit?\n%s" % (self.wc, msg, err))
+            self.log.warning("Commit from working copy %s (msg: %s) failed, empty commit?\n%s" % (self.wc, msg, err))
         try:
             info = self.client.push()
-            log.debug("push info: %s " % info)
+            self.log.debug("push info: %s " % info)
         except GitCommandError, err:
-            log.warning("Push from working copy %s to remote %s (msg: %s) failed: %s" % (self.wc, self.repo, msg, err))
+            self.log.warning("Push from working copy %s to remote %s (msg: %s) failed: %s" % (self.wc,
+                                                                                              self.repo,
+                                                                                              msg,
+                                                                                              err))
 
     def cleanup(self):
         """
@@ -298,7 +325,7 @@ class GitRepository(FileRepository):
         try:
             rmtree2(self.wc)
         except IOError, err:
-            log.exception("Can't remove working copy %s: %s" % (self.wc, err))
+            self.log.exception("Can't remove working copy %s: %s" % (self.wc, err))
 
 
 class SvnRepository(FileRepository):
@@ -306,6 +333,11 @@ class SvnRepository(FileRepository):
     Class for svn repositories
     """
 
+    DESCRIPTION = ("A SVN repository. The 1st argument contains the "
+                   "subversion repository location, this can be a directory or an URL.")
+
+    USABLE = HAVE_PYSVN
+
     def __init__(self, *args):
         """
         Set self.client to None. Real logic is in setupRepo and createWorkingCopy
@@ -319,24 +351,24 @@ class SvnRepository(FileRepository):
         """
         self.repo = os.path.join(self.repo, self.subdir)
         try:
-            raise pysvn.ClientError #IGNORE:E0611 pysvn fails to recognize ClientError is available
+            raise pysvn.ClientError  # IGNORE:E0611 pysvn fails to recognize ClientError is available
         except NameError, err:
-            log.exception("pysvn not available (%s). Make sure it is installed " % err +
+            self.log.exception("pysvn not available (%s). Make sure it is installed " % err +
                           "properly. Run 'python -c \"import pysvn\"' to test.")
 
-        ## try to connect to the repository
-        log.debug("Try to connect to repository %s" % self.repo)
+        # try to connect to the repository
+        self.log.debug("Try to connect to repository %s" % self.repo)
         try:
             self.client = pysvn.Client()
             self.client.exception_style = 0
         except ClientError:
-            log.exception("Svn Client initialization failed.")
+            self.log.exception("Svn Client initialization failed.")
 
         try:
             if not self.client.is_url(self.repo):
-                log.error("Provided repository %s is not a valid svn url" % self.repo)
+                self.log.error("Provided repository %s is not a valid svn url" % self.repo)
         except ClientError:
-            log.exception("Can't connect to svn repository %s" % self.repo)
+            self.log.exception("Can't connect to svn repository %s" % self.repo)
 
     def create_working_copy(self):
         """
@@ -344,55 +376,56 @@ class SvnRepository(FileRepository):
         """
         self.wc = tempfile.mkdtemp(prefix='svn-wc-')
 
-        ## check if tmppath exists
-        ## this will trigger an error if it does not exist
+        # check if tmppath exists
+        # this will trigger an error if it does not exist
         try:
             self.client.info2(self.repo, recurse=False)
         except ClientError:
-            log.exception("Getting info from %s failed." % self.wc)
+            self.log.exception("Getting info from %s failed." % self.wc)
 
         try:
             res = self.client.update(self.wc)
-            log.debug("Updated to revision %s in %s" % (res, self.wc))
+            self.log.debug("Updated to revision %s in %s" % (res, self.wc))
         except ClientError:
-            log.exception("Update in wc %s went wrong" % self.wc)
+            self.log.exception("Update in wc %s went wrong" % self.wc)
 
         if len(res) == 0:
-            log.error("Update returned empy list (working copy: %s)" % (self.wc))
+            self.log.error("Update returned empy list (working copy: %s)" % (self.wc))
 
         if res[0].number == -1:
-            ## revision number of update is -1
-            ## means nothing has been checked out
+            # revision number of update is -1
+            # means nothing has been checked out
             try:
                 res = self.client.checkout(self.repo, self.wc)
-                log.debug("Checked out revision %s in %s" % (res.number, self.wc))
+                self.log.debug("Checked out revision %s in %s" % (res.number, self.wc))
             except ClientError, err:
-                log.exception("Checkout of path / in working copy %s went wrong: %s" % (self.wc, err))
+                self.log.exception("Checkout of path / in working copy %s went wrong: %s" % (self.wc, err))
 
     def add_easyconfig(self, cfg, name, version, stats, append):
         """
         Add easyconfig to SVN repository.
         """
         dest = FileRepository.add_easyconfig(self, cfg, name, version, stats, append)
-        log.debug("destination = %s" % dest)
+        self.log.debug("destination = %s" % dest)
         if dest:
-            log.debug("destination status: %s" % self.client.status(dest))
+            self.log.debug("destination status: %s" % self.client.status(dest))
 
             if self.client and not self.client.status(dest)[0].is_versioned:
-                ## add it to version control
-                log.debug("Going to add %s (working copy: %s, cwd %s)" % (dest, self.wc, os.getcwd()))
+                # add it to version control
+                self.log.debug("Going to add %s (working copy: %s, cwd %s)" % (dest, self.wc, os.getcwd()))
                 self.client.add(dest)
 
-
     def commit(self, msg=None):
         """
         Commit working copy to SVN repository
         """
-        completemsg = "EasyBuild-commit from %s (time: %s, user: %s) \n%s" % (socket.gethostname(), time.strftime("%Y-%m-%d_%H-%M-%S"), getpass.getuser(), msg)
+        completemsg = "EasyBuild-commit from %s (time: %s, user: %s) \n%s" % (socket.gethostname(),
+                                                                              time.strftime("%Y-%m-%d_%H-%M-%S"),
+                                                                              getpass.getuser(), msg)
         try:
             self.client.checkin(self.wc, completemsg, recurse=True)
         except ClientError, err:
-            log.exception("Commit from working copy %s (msg: %s) failed: %s" % (self.wc, msg, err))
+            self.log.exception("Commit from working copy %s (msg: %s) failed: %s" % (self.wc, msg, err))
 
     def cleanup(self):
         """
@@ -401,5 +434,17 @@ class SvnRepository(FileRepository):
         try:
             rmtree2(self.wc)
         except OSError, err:
-            log.exception("Can't remove working copy %s: %s" % (self.wc, err))
+            self.log.exception("Can't remove working copy %s: %s" % (self.wc, err))
 
+
+def get_repositories(check_usable=True):
+    """
+    Return all repositories.
+        check_usable: boolean, if True, only return usable repositories
+    """
+    class_dict = dict([(x.__name__, x) for x in get_subclasses(Repository) if x.USABLE or not check_usable])
+
+    if not 'FileRepository' in class_dict:
+        _log.error('get_repositories: FileRepository missing from list of repositories')
+
+    return class_dict
